/*
 * Copyright (C) 2003-2005  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define CACHE_INTERNAL_ACCESS
#include "include.h"
#include "cache.h"
#include "conf.h"
#include "globals.h"
#include "spell.h"
#include "targy.h"
#include "fordulo.h"
#include "input.h"
#include "format.h"
#include "win.h"

static t_cache_conf cache_conf[] =
{
    {
	cache_type_monster, 
	0xbaddadee, 
	"_szorny.dat", 
	sizeof(t_cache_monster)/sizeof(int), 
	cache_add_header_monster, 
	cache_add_monster, 
	cache_get_header_monster, 
	cache_get_monster, 
	cache_get_monster_max_ssz
    },
    
    {
	cache_type_item, 
	0xbaddadef, 
	"_targy.dat", 
	sizeof(t_cache_item)/sizeof(int), 
	cache_add_header_item, 
	cache_add_item, 
	cache_get_header_item, 
	cache_get_item, 
	cache_get_item_max_ssz
    },
    
    {
	cache_type_spell, 
	0xbaddadea, 
	"_varazslat.dat", 
	sizeof(t_cache_spell)/sizeof(int), 
	cache_add_header_spell, 
	cache_add_spell, 
	cache_get_header_spell, 
	cache_get_spell, 
	cache_get_spell_max_ssz
    },

    {
	cache_type_stat, 
	0xbaddadeb, 
	"_stat.dat", 
	sizeof(t_cache_stat)/sizeof(int), 
	cache_add_header_stat, 
	cache_add_stat, 
	cache_get_header_stat, 
	cache_get_stat, 
	cache_get_stat_max_ssz
    }
};

static int cachedebug = 0;
static cache_build_ksz = 0;

static t_cache_work cache_work[] =
{
    { "build", cache_build },
    { "update", cache_update }
};

static t_cache_job * cache_job_first = NULL;
static t_cache_job * cache_job_last = NULL;

int conf_i(t_cache_type type)
{
    int i;
    
    for (i = 0; i < sizeof(cache_conf)/sizeof(t_cache_conf); i++)
    {
	if (cache_conf[i].type == type)
	{
	    return i;
	}
    }
    
    return -1;
}

char * fsz_arr_create(high_fsz)
{
    char * fsz_arr;

    if (!(fsz_arr = malloc(high_fsz+1)))
    {
	return NULL;;
    }
    memset(fsz_arr, 0, high_fsz+1);

    return fsz_arr;
}

extern get_current_cache_version(t_cache_type type)
{
    t_cache_version cache_version;
    
    switch(type)
    {
	case cache_type_monster:
	    cache_version = cache_version_1;
	    break;
	case cache_type_item:
	    cache_version = cache_version_1;
	    break;
	case cache_type_spell:
	    cache_version = cache_version_1;
	    break;
	case cache_type_stat:
	    cache_version = cache_version_3;
	    break;
	default:
	    cache_version = cache_version_none;
	    break;
    }
    
    return cache_version;
}

extern int cache_add_monster(t_cache * cache, int * buf, int n)
{
    int ssz;
    int i;
    int * ii;
    int max_fsz;
    t_cache_monster * monster;
    
    max_fsz = 0;
    ssz = buf[0];

    if (!cache)
    {
	return -1;
    }

    if ((cache->monster_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    if (cache->monsters[ssz].ssz == ssz)
    {
	if (cache->monsters[ssz].fsz > buf[1])
	{
	    cache->monsters[ssz].fsz = buf[1];
	    cache->monsters[ssz].flag = buf[2];
	}
	return 0;
    }
    
    monster = &cache->monsters[ssz];
    memset(monster, 0, sizeof(t_cache_monster));
    
    ii = (int *)monster;
    ii[0] = ssz;
    
#ifdef DEBUG
    debug(NULL, "cache_add_monster: ssz=\"%d\", params=%d\n", ssz, n);
#endif
    for (i = 1; (i < n) && (i < (sizeof(t_cache_monster) / sizeof(int))); i++)
    {
#ifdef DEBUG
        debug(NULL, "cache_add_monster: param[%d]=\"%d\"\n", i, buf[i]);
#endif
        ii[i] = buf[i];
        if ((i < (sizeof(t_cache_item)/sizeof(int)-2)) && (i%2 == 1) && (max_fsz < buf[i]))
        {
	    max_fsz = buf[i];
	}
    }
    
    if (!cache->monster_header && (max_fsz >= 0))
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(max_fsz))
	{
	    fsz_arr[max_fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_monster);
	    cache_add_header_monster(cache, cache_version, max_fsz, fsz_arr);
	}
    }
    
    return 0;
}

extern int cache_add_item(t_cache * cache, int * buf, int n)
{
    int ssz;
    int i;
    int * ii;
    int max_fsz;
    t_cache_item * item;
    
    ssz = buf[0];
    max_fsz = 0;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->item_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    if (cache->items[ssz].ssz == ssz)
    {
	if (cache->items[ssz].fsz > buf[1])
	{
	    cache->items[ssz].fsz = buf[1];
	    cache->items[ssz].flag = buf[2];
	}
	return 0;
    }
    
    item = &cache->items[ssz];
    memset(item, 0, sizeof(t_cache_item));
    
    ii = (int *)item;
    ii[0] = ssz;
    
#ifdef DEBUG
    debug(NULL, "cache_add_item: ssz=\"%d\", params=%d\n", ssz, n);
#endif
    for (i = 1; (i < n) && (i < (sizeof(t_cache_item) / sizeof(int))); i++)
    {
#ifdef DEBUG
        debug(NULL, "cache_add_item: param[%d]=\"%d\"\n", i, buf[i]);
#endif
        ii[i] = buf[i];
        if ((i < (sizeof(t_cache_item)/sizeof(int))) && (i%2 == 1) && (max_fsz < buf[i]))
        {
	    max_fsz = buf[i];
	}
    }
    
    if (!cache->item_header && (max_fsz >= 0))
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(max_fsz))
	{
	    fsz_arr[max_fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_item);
	    cache_add_header_item(cache, cache_version, max_fsz, fsz_arr);
	}
    }
    
    return 0;
}

extern int cache_add_spell(t_cache * cache, int * buf, int n)
{
    int ssz;
    int i;
    int * ii;
    int max_fsz;
    t_cache_spell * spell;
    
    ssz = buf[0];
    max_fsz = 0;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->spell_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    if (cache->spells[ssz].ssz == ssz)
    {
	if (cache->spells[ssz].fsz > buf[1])
	{
	    cache->spells[ssz].fsz = buf[1];
	    cache->spells[ssz].flag = buf[2];
	    memcpy(&cache->spells[ssz].count, &buf[3], (n - 3 < sizeof(long) ? n - 3 : sizeof(long)));
	}
	return 0;
    }
    
    spell = &cache->spells[ssz];
    memset(spell, 0, sizeof(t_cache_spell));
    
    ii = (int *)spell;
    ii[0] = ssz;
    
#ifdef DEBUG
    debug(NULL, "cache_add_spell: ssz=\"%d\", params=%d\n", ssz, n);
#endif
    for (i = 1; (i < n) && (i < ((sizeof(t_cache_spell) - sizeof(long)) / sizeof(int))); i++)
    {
#ifdef DEBUG
        debug(NULL, "cache_add_spell: param[%d]=\"%d\"\n", i, buf[i]);
#endif
        ii[i] = buf[i];
        if ((i < (sizeof(t_cache_spell) - sizeof(long))/sizeof(int)) && (i%2 == 1) && (max_fsz < buf[i]))
        {
	    max_fsz = buf[i];
	}
    }
    
    if (!cache->spell_header && (max_fsz >= 0))
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(max_fsz))
	{
	    fsz_arr[max_fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_spell);
	    cache_add_header_spell(cache, cache_version, max_fsz, fsz_arr);
	}
    }
    
    return 0;
}

extern int cache_add_stat(t_cache * cache, int * buf, int n)
{
    int ssz;
    int i;
    int * ii;
    int max_fsz;
    t_cache_stat * stat;
    
    ssz = buf[0];
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->stat_max_ssz < ssz) || (ssz < 0))
    {
	return -2;
    }

    stat = &cache->stats[ssz];
    memset(stat, 0, sizeof(t_cache_stat));
    
    ii = (int *)stat;
    ii[0] = ssz;
    
#ifdef DEBUG
    debug(NULL, "cache_add_stat: ssz=\"%d\", params=%d\n", ssz, n);
#endif
    for (i = 1; (i < n) && (i < (sizeof(t_cache_stat) / sizeof(int))); i++)
    {
#ifdef DEBUG
        debug(NULL, "cache_add_stat: param[%d]=\"%d\"\n", i, buf[i]);
#endif
        ii[i] = buf[i];
        if ((i < (sizeof(t_cache_stat) - sizeof(long))/sizeof(int)) && (i%2 == 1) && (max_fsz < buf[i]))
        {
	    max_fsz = buf[i];
	}
    }
    
    if (!cache->stat_header && (max_fsz >= 0))
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(max_fsz))
	{
	    fsz_arr[max_fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_stat);
	    cache_add_header_stat(cache, cache_version, max_fsz, fsz_arr);
	}
    }
    
    return 0;
}

extern int cache_get_monster(int ssz, t_cache * cache, int * result)
{
    int confi;
    int count;
    int * ii;
    int ret;
    
    ret = -1;
    
    if (!result)
    {
	return ret;
    }
    
    if ((confi = conf_i(cache_type_monster)) == -1)
    {
	return ret;
    }
    
    if ((ssz >= 1) && (ssz <= (* cache_conf[confi].cache_get_max_ssz)(cache)))
    {
	ii = (int *)&cache->monsters[ssz];
	
	for (count = 0; count < cache_conf[confi].paramcount; count++)
	{
	    result[count] = ii[count];
	}
	
	ret = 0;
    }
    
    return ret;
}

extern int cache_get_item(int ssz, t_cache * cache, int * result)
{
    int confi;
    int count;
    int * ii;
    int ret;
    
    ret = -1;
    
    if (!result)
    {
	return ret;
    }
    
    if ((confi = conf_i(cache_type_item)) == -1)
    {
	return ret;
    }
    
    if ((ssz >= 1) && (ssz <= (* cache_conf[confi].cache_get_max_ssz)(cache)))
    {
	ii = (int *)&cache->items[ssz];
	
	for (count = 0; count < cache_conf[confi].paramcount; count++)
	{
	    result[count] = ii[count];
	}
	
	ret = 0;
    }
    
    return ret;
}

extern int cache_get_spell(int ssz, t_cache * cache, int * result)
{
    int confi;
    int count;
    int * ii;
    int ret;
    
    ret = -1;
    
    if (!result)
    {
	return ret;
    }
    
    if ((confi = conf_i(cache_type_spell)) == -1)
    {
	return ret;
    }
    
    if ((ssz >= 1) && (ssz <= (* cache_conf[confi].cache_get_max_ssz)(cache)))
    {
	ii = (int *)&cache->spells[ssz];
	
	for (count = 0; count < cache_conf[confi].paramcount; count++)
	{
	    result[count] = ii[count];
	}
	
	ret = 0;
    }
    
    return ret;
}

extern int cache_get_stat(int ssz, t_cache * cache, int * result)
{
    int confi;
    int count;
    int * ii;
    int ret;
    
    ret = -1;
    
    if (!result)
    {
	return ret;
    }
    
    if ((confi = conf_i(cache_type_stat)) == -1)
    {
	return ret;
    }
    
    if ((ssz >= 1) && (ssz <= (* cache_conf[confi].cache_get_max_ssz)(cache)))
    {
	ii = (int *)&cache->stats[ssz];
	
	for (count = 0; count < cache_conf[confi].paramcount; count++)
	{
	    result[count] = ii[count];
	}
	
	ret = 0;
    }
    
    return ret;
}


int cache_load_file(t_cache_type type, t_cache * cache, char * filename, int fsz)
{
    FILE * fp = NULL;
    int confi;
    int ret = -1;
    int checksum, count;
    int start;
    int high_fsz;
    char * fsz_arr = NULL;
    int * buf;
    int buf_len;
    t_cache_version cache_version;
    
#ifdef DEBUG
    debug(NULL, "cache_load_file: fsz=%d, filename=\"%s\"\n", fsz, filename);
#endif
    
    if ((confi = conf_i(type)) == -1)
    {
	return ret;
    }

    buf_len = cache_conf[confi].paramcount * sizeof(int) + 1;
    if (!(buf = malloc(buf_len)))
    {
	return ret;
    }
    memset(buf, 0, buf_len);
    
    for (checksum = 0; cache_conf[confi].paramcount && cache && filename; )
    {
	checksum = 0;

	if (!(fp = fopen(filename, "rb")))
	{
	    break;
	}
	
	if ((fread(buf, sizeof(int), 3, fp) != 3) ||  (buf[0] != cache_conf[confi].magick))
	{
	    break;
	}
	
	checksum += buf[1];
	cache_version = (t_cache_version)buf[1];
	
	if (cache_version != get_current_cache_version(type))
	{
	    kiir(NULL, "\nNem jo cache verzio, torold es epitsd ujra: \"%s\".\n", filename);
	    ret = -2;
	    break;
	}
	
	checksum += buf[2];
	high_fsz = (fsz > buf[2] ? fsz : buf[2]);

	if (!(fsz_arr = fsz_arr_create(high_fsz)))
	{
	    break;
	}
	
	if (fread(fsz_arr, sizeof(char), buf[2]+1, fp) != buf[2]+1)
	{
	    free(fsz_arr);
	    fsz_arr = NULL;
	    break;
	}
	
	for (count = 0; count < buf[2]+1; count++) checksum += fsz_arr[count];

	if (fsz_arr[fsz] == ((char)cache_fsz_flag_none))
	{
	    fsz_arr[fsz] = (char)cache_fsz_flag_init;
	}
	else if (fsz_arr[fsz] != ((char)cache_fsz_flag_saved))
	{
	    break;
	}
	
	(* cache_conf[confi].cache_add_header)(cache, cache_version, high_fsz, fsz_arr);
	while (fread(buf, sizeof(int), cache_conf[confi].paramcount, fp) == cache_conf[confi].paramcount)
	{
	    (* cache_conf[confi].cache_add)(cache, buf, buf_len/sizeof(int));
	    for (count = 0; count < cache_conf[confi].paramcount; count++) checksum += buf[count];
	}
	
	fread(buf, sizeof(int), 1, fp);
	
	if (feof(fp) == 0)
	{
	    break;
	}
#ifdef DEBUG
	debug(NULL, "cache_load_file: checksum: buf[0]=%d, checksum=%d\n", buf[0], checksum);
#endif
	if (buf[0] != checksum)
	{
	    break;
	}
	
	ret = 0;
	break;
    }
    
    if (fp)
    {
	fclose(fp);
    }
    if (buf)
    {
	free(buf);
    }
    return ret;
}

int cache_save_file(t_cache_type type, t_cache * cache, char * filename, int fsz)
{
    FILE * fp = NULL;
    int confi;
    int ret = -1;
    int checksum, count;
    int ssz;
    char * fsz_arr = NULL;
    int * buf;
    int buf_len;
    t_cache_version cache_version;
    
#ifdef DEBUG
    debug(NULL, "cache_save_file: filename=\"%s\", type=%d\n", filename, type);
#endif

    if ((confi = conf_i(type)) == -1)
    {
	return ret;
    }
    
    buf_len = cache_conf[confi].paramcount * sizeof(int) + 1;
    if (!(buf = malloc(buf_len)))
    {
	return ret;
    }
    memset(buf, 0, buf_len);
    
    for (checksum = 0; cache_conf[confi].paramcount && cache && filename; )
    {
	buf[0] = cache_conf[confi].magick;
	
	if (!(fsz_arr = (* cache_conf[confi].cache_get_header)(cache, &buf[2], &buf[1])))
	{
	    break;
	}
	
	if ((fsz_arr[fsz] != (char)cache_fsz_flag_init))
	{
	    break;
	}
	
	if (!(fp = fopen(filename, "wb")))
	{
	    break;
	}
    
	fsz_arr[fsz] = (char)cache_fsz_flag_saved;
	
	fwrite(buf, sizeof(int), 3, fp);
	fwrite(fsz_arr, sizeof(char), buf[2]+1, fp);
#ifdef DEBUG
	debug(NULL, "cache_save_file: buf= %d, %d\n", buf[0], buf[2]);
#endif
	checksum += buf[1];
	checksum += buf[2];
	for (count = 0; count < buf[2]+1; count++) checksum += fsz_arr[count];
	
	for(ssz = 1; ((* cache_conf[confi].cache_get)(ssz, cache, buf) != -1); ssz++)
	{
	    if (buf[0] != 0)
	    {
		fwrite(buf, sizeof(int), cache_conf[confi].paramcount, fp);
#ifdef DEBUG
		debug(NULL, "cache_save_file: param[%d]= %d, darab=%d\n", ssz, buf[0], cache_conf[confi].paramcount);
#endif
		for (count = 0; count < cache_conf[confi].paramcount; count++) checksum += buf[count];
	    }
	}

	buf[0] = checksum;
	fwrite(buf, sizeof(int), 1, fp);
#ifdef DEBUG
	debug(NULL, "cache_save_file: checksum= %d\n", buf[0]);
#endif
	
	ret = 0;
	break;
    }
    
    if (fp)
    {
	fclose(fp);
    }
    if (buf)
    {
	free(buf);
    }
    return ret;
}

extern int cache_load(t_fordulo * fordulo)
{
    t_cache * cache = NULL;
    char * file = NULL;
    char * filename = NULL;
    char * cachedir = NULL;
    int hiba;
    int r;
    int i;
    
    hiba = 0;
    
    if (!fordulo || fordulo->cache)
    {
	return -1;
    }
    
    if (fordulo->karakter->ksz && (cache = cache_create(fordulo)))
    {
	cachedir = bconf_get_cachedir();
	
	for (i = 0; i < sizeof(cache_conf)/sizeof(t_cache_conf); i++)
	{
	    if (file = malloc(4 + strlen(cache_conf[i].name) + 1))
	    {
		sprintf(file, "%4d%s", fordulo->karakter->ksz, cache_conf[i].name);
		filename = get_filename(cachedir, file);
		free(file);
		
		r = cache_load_file(cache_conf[i].type, cache, filename, fordulo->fsz);
		
		free(filename);
		filename = NULL;
		if (r == -2)
		{
		    hiba = 1;
		    break;
		}
		hiba += r;
	    }
	}
    }
    else
    {
	hiba = -1;
    }
    
    if (hiba == i)
    {
	if (cache)
	{
	    cache_destroy(cache);
	    cache = NULL;
	}
    }

    fordulo->cache = cache;    

    return hiba;
}


extern int cache_save(t_fordulo * fordulo)
{
    t_cache * cache = NULL;
    char * file = NULL;
    char * filename = NULL;
    char * cachedir = NULL;
    int hiba;
    int i;
    
    hiba = 0;
    
    if (!fordulo || !fordulo->cache)
    {
	return -1;
    }
    cache = fordulo->cache;
    cache_dump(fordulo, cache);
    
    if (fordulo->karakter->ksz)
    {
	cachedir = bconf_get_cachedir();
	
	for (i = 0; i < sizeof(cache_conf)/sizeof(t_cache_conf); i++)
	{
	    if (file = malloc(4 + strlen(cache_conf[i].name) + 1))
	    {
		sprintf(file, "%4d%s", fordulo->karakter->ksz, cache_conf[i].name);
		filename = get_filename(cachedir, file);
		free(file);
		
		hiba += cache_save_file(cache_conf[i].type, cache, filename, fordulo->fsz);
		
		free(filename);
		filename = NULL;
	    }
	}
    }
    else
    {
	hiba = -1;
    }
    
    return hiba;
}

extern int cache_unload(t_fordulo * fordulo)
{
    if (fordulo)
    {
	cache_destroy(fordulo->cache);
	fordulo->cache = NULL;
    }
        
    return 0;
}

extern int cache_add_monster_megoles(t_cache * cache, int ssz, t_cache_monster_megoli_flag flag)
{
    int fsz;
    t_cache_monster * monster;
    int i;
    
    fsz = 0;
    i = 0;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->monster_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }

    fsz = cache->fsz;
    
    if (!cache->monster_header)
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(fsz))
	{
	    fsz_arr[fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_monster);
	    cache_add_header_monster(cache, cache_version, fsz, fsz_arr);
	}
    }
    
    if ((cache->monster_header->high_fsz < fsz))
    {
	return -1;
    }
    
    if (cache->monster_header->fsz_arr[fsz] != (char)cache_fsz_flag_init)
    {
	return 0;
    }
    
    monster = &cache->monsters[ssz];

#ifdef DEBUG
    debug(NULL, "cache_add_monster_megoles: ssz=\"%d\", flag=%d\n", ssz, flag);
#endif
    
    if (monster->ssz == 0)
    {
	memset(monster, 0, sizeof(t_cache_monster));
	monster->ssz = ssz;
	monster->fsz = fsz;
	monster->flag = cache_monster_flag_ismert;
	monster->megoles[0].fsz = fsz;
	monster->megoles[0].flag = (int)flag;
	if (flag&cache_monster_megoli_flag_megoli)
	{
	    monster->count_megolt++;
	}
	else if (flag&cache_monster_megoli_flag_nemoli)
	{
	    monster->count_nemolt++;
	}
    }
    else if (monster->ssz == ssz)
    {
	for (i = CACHE_MONSTER_MEGOLES_LEN - 1; i > 0; i--)
	{
	    if (monster->megoles[i-1].fsz)
	    {
		if (monster->megoles[i-1].fsz <= fsz)
		{
		    monster->megoles[i].fsz = monster->megoles[i-1].fsz;
		    monster->megoles[i].flag = monster->megoles[i-1].flag;
		    monster->megoles[i-1].fsz = 0;
		    monster->megoles[i-1].flag = (int)cache_monster_megoli_flag_none;
		}
		else
		{
		    break;
		}
	    }
	}
	if (monster->megoles[i].fsz <= fsz)
	{
	    monster->megoles[i].fsz = fsz;
	    monster->megoles[i].flag = (int)flag;
	    if (flag&cache_monster_megoli_flag_megoli)
	    {
		monster->count_megolt++;
	    }
	    else if (flag&cache_monster_megoli_flag_nemoli)
	    {
		monster->count_nemolt++;
	    }
	}
    }
    
    return 0;
}

extern int cache_add_spell_count(t_fordulo * fordulo, t_cache * cache, int ssz)
{
    int fsz;
    t_cache_spell * spell;
    int i;
    int only_curr;
    
    fsz = 0;
    i = 0;
    only_curr = 0;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->spell_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }

    fsz = cache->fsz;
    
    if (!cache->spell_header)
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(fsz))
	{
	    fsz_arr[fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_spell);
	    cache_add_header_spell(cache, cache_version, fsz, fsz_arr);
	}
    }
    
    if ((cache->spell_header->high_fsz < fsz))
    {
	return -1;
    }
    
    if (cache->spell_header->fsz_arr[fsz] != (char)cache_fsz_flag_init)
    {
	only_curr = 1;;
    }
    
    spell = &cache->spells[ssz];

#ifdef DEBUG
    debug(NULL, "cache_add_spell_count: ssz=\"%d\"\n", ssz);
#endif
    
    if (spell->ssz == 0)
    {
	memset(spell, 0, sizeof(t_cache_spell));
	spell->ssz = ssz;
	spell->fsz = fsz;
	spell->flag = cache_spell_flag_ismert;
	spell->curr = 1;
	spell->count = 1;
    }
    else if (spell->ssz == ssz)
    {
	spell->curr++;
	if (!only_curr)
	{
	    spell->count++;
	}
    }
    
    if (fordulo && fordulo->karakter)
    {
	if (ssz == 12)
	{
	    fordulo->karakter->eropajzs = 2;
	}
	else if (ssz == 63)
	{
	    fordulo->karakter->eropajzs = 4;
	}
	else if (ssz == 127)
	{
	    fordulo->karakter->eropajzs = 6;
	}
    }
    return 0;
}

extern int cache_add_stat_value(t_cache * cache, int ssz, long value)
{
    int fsz;
    t_cache_stat * stat;
    int i;
    
    fsz = 0;
    i = 0;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->stat_max_ssz < ssz) || (ssz < 0))
    {
	return -2;
    }

    fsz = cache->fsz;
    
    if (!cache->stat_header)
    {
	char * fsz_arr;
	t_cache_version cache_version;
	
	if (fsz_arr = fsz_arr_create(fsz))
	{
	    fsz_arr[fsz] = (char)cache_fsz_flag_init;
	    cache_version = get_current_cache_version(cache_type_stat);
	    cache_add_header_stat(cache, cache_version, fsz, fsz_arr);
	}
    }
    
    if ((cache->stat_header->high_fsz < fsz))
    {
	return -1;
    }
    
    if (cache->stat_header->fsz_arr[fsz] != (char)cache_fsz_flag_init)
    {
	return 0;
    }
    
    stat = &cache->stats[ssz];

#ifdef DEBUG
    debug(NULL, "cache_add_stat_value: ssz=\"%d\", old=\"%ld\", add=\"%ld\", new=\"%ld\"\n", ssz, stat->value, value, stat->value+value);
#endif
    
//    if (stat->ssz == ssz)
    {
	stat->ssz = ssz;
	stat->fsz = fsz;
	stat->value += value;
    }
    
    return 0;
}

extern int cache_add_header_monster(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr)
{
    t_cache_header * monster_header;
    
    monster_header = NULL;
    
    if (!cache || !fsz_arr)
    {
	return -1;
    }
    
    if (!(monster_header = malloc(sizeof(t_cache_header))))
    {
	return -1;
    }
    
#ifdef DEBUG
    debug(NULL, "cache_add_header_monster: fsz=%d, cache->monster_header=%p, high_fsz=%d, fsz_arr=%p\n", cache->fsz, cache->monster_header, high_fsz, fsz_arr);
#endif
    
    monster_header->cache_version = cache_version;
    monster_header->high_fsz = high_fsz;
    monster_header->fsz_arr = fsz_arr;
    
    if (cache->monster_header)
    {
	if (cache->monster_header->high_fsz < high_fsz)
	{
	    free(cache->monster_header->fsz_arr);
	    free(cache->monster_header);
	}
	else
	{
	    cache->monster_header->fsz_arr[cache->fsz] = fsz_arr[cache->fsz];
	    free(monster_header);
	    monster_header = NULL;
	}
    }
    
    if (monster_header)
    {
	cache->monster_header = monster_header;
    }
    
    return 0;
}

extern int cache_add_header_item(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr)
{
    t_cache_header * item_header;
    
    item_header = NULL;
    
    if (!cache || !fsz_arr)
    {
	return -1;
    }
    
    if (!(item_header = malloc(sizeof(t_cache_header))))
    {
	return -1;
    }
    
#ifdef DEBUG
    debug(NULL, "cache_add_header_item: fsz=%d, cache->item_header=%p, high_fsz=%d, fsz_arr=%p\n", cache->fsz, cache->item_header, high_fsz, fsz_arr);
#endif    
    
    item_header->cache_version = cache_version;
    item_header->high_fsz = high_fsz;
    item_header->fsz_arr = fsz_arr;
    
    if (cache->item_header)
    {
	if (cache->item_header->high_fsz < high_fsz)
	{
	    free(cache->item_header->fsz_arr);
	    free(cache->item_header);
	}
	else
	{
	    cache->item_header->fsz_arr[cache->fsz] = fsz_arr[cache->fsz];
	    free(item_header);
	    item_header = NULL;
	}
    }
    
    if (item_header)
    {
	cache->item_header = item_header;
    }
    
    return 0;
}

extern int cache_add_header_spell(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr)
{
    t_cache_header * spell_header;
    
    spell_header = NULL;
    
    if (!cache || !fsz_arr)
    {
	return -1;
    }
    
    if (!(spell_header = malloc(sizeof(t_cache_header))))
    {
	return -1;
    }
    
#ifdef DEBUG
    debug(NULL, "cache_add_header_spell: fsz=%d, cache->spell_header=%p, high_fsz=%d, fsz_arr=%p\n", cache->fsz, cache->spell_header, high_fsz, fsz_arr);
#endif    
    
    spell_header->cache_version = cache_version;
    spell_header->high_fsz = high_fsz;
    spell_header->fsz_arr = fsz_arr;
    
    if (cache->spell_header)
    {
	if (cache->spell_header->high_fsz < high_fsz)
	{
	    free(cache->spell_header->fsz_arr);
	    free(cache->spell_header);
	}
	else
	{
	    cache->spell_header->fsz_arr[cache->fsz] = fsz_arr[cache->fsz];
	    free(spell_header);
	    spell_header = NULL;
	}
    }
    
    if (spell_header)
    {
	cache->spell_header = spell_header;
    }
    
    return 0;
}

extern int cache_add_header_stat(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr)
{
    t_cache_header * stat_header;
    
    stat_header = NULL;
    
    if (!cache || !fsz_arr)
    {
	return -1;
    }
    
    if (!(stat_header = malloc(sizeof(t_cache_header))))
    {
	return -1;
    }
    
#ifdef DEBUG
    debug(NULL, "cache_add_header_stat: fsz=%d, cache->stat_header=%p, high_fsz=%d, fsz_arr=%p\n", cache->fsz, cache->stat_header, high_fsz, fsz_arr);
#endif    
    
    stat_header->cache_version = cache_version;
    stat_header->high_fsz = high_fsz;
    stat_header->fsz_arr = fsz_arr;
    
    if (cache->stat_header)
    {
	if (cache->stat_header->high_fsz < high_fsz)
	{
	    free(cache->stat_header->fsz_arr);
	    free(cache->stat_header);
	}
	else
	{
	    cache->stat_header->fsz_arr[cache->fsz] = fsz_arr[cache->fsz];
	    free(stat_header);
	    stat_header = NULL;
	}
    }
    
    if (stat_header)
    {
	cache->stat_header = stat_header;
    }
    
    return 0;
}

extern char * cache_get_header_monster(t_cache * cache, int * buf, int * ver)
{
    int x;
    
    x = 0;
    
    if (cache && cache->monster_header && buf && ver)
    {
	buf[0] = cache->monster_header->high_fsz;
	ver[0] = cache->monster_header->cache_version;
	return cache->monster_header->fsz_arr;
    }
    return NULL;
}

extern char * cache_get_header_item(t_cache * cache, int * buf, int * ver)
{
    int x;
    
    x = 0;
    
    if (cache && cache->item_header && buf && ver)
    {
	buf[0] = cache->item_header->high_fsz;
	ver[0] = cache->item_header->cache_version;
	return cache->item_header->fsz_arr;
    }
    return NULL;
}

extern char * cache_get_header_spell(t_cache * cache, int * buf, int * ver)
{
    int x;
    
    x = 0;
    
    if (cache && cache->spell_header && buf && ver)
    {
	buf[0] = cache->spell_header->high_fsz;
	ver[0] = cache->spell_header->cache_version;
	return cache->spell_header->fsz_arr;
    }
    return NULL;
}

extern char * cache_get_header_stat(t_cache * cache, int * buf, int * ver)
{
    int x;
    
    x = 0;
    
    if (cache && cache->stat_header && buf)
    {
	buf[0] = cache->stat_header->high_fsz;
	ver[0] = cache->stat_header->cache_version;
	return cache->stat_header->fsz_arr;
    }
    return NULL;
}

extern int cache_check_monster(t_fordulo * fordulo, t_cache * cache, int ssz, int * percent, int last_n, int * last_a)
{
    t_cache_monster * monster;
    int i;
    int megoli, nemoli;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->monster_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    
    monster = &cache->monsters[ssz];
    if (monster)
    {
	if ((monster->ssz != ssz) || (monster->fsz > fordulo->fsz))
	{
	    monster = NULL;
	}
    }
    
    if (monster)
    {
	for (megoli = 0, nemoli = 0, i = CACHE_MONSTER_MEGOLES_LEN - 1; i >= 0; i--)
	{
	    if (monster->megoles[i].fsz)
	    {
		if (monster->megoles[i].flag == (int)cache_monster_megoli_flag_megoli)
		{
		    megoli++;
		}
		else if (monster->megoles[i].flag == (int)cache_monster_megoli_flag_nemoli)
		{
		    nemoli++;
		}
		
		if (last_a && (last_n > i))
		{
		    last_a[i*2] = monster->megoles[i].fsz;
		    last_a[i*2+1] = monster->megoles[i].flag;
		}
	    }
	}
	
	if (percent)
	{
	    if (megoli + nemoli)
	    {
		*percent = 100 * megoli / (megoli + nemoli);
	    }
	    else
	    {
		*percent = 0;
	    }
	}
	return (int)cache_monster_flag_ismert;
    }
    
    return (int)cache_monster_flag_none;
}

extern int cache_check_item(t_fordulo * fordulo, t_cache * cache, int ssz)
{
    t_cache_item * item;
    
    if (!cache)
    {
	return -1;
    }

    if ((cache->item_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    
    item = &cache->items[ssz];
    
    if (item)
    {
	if ((item->ssz != ssz) || (item->fsz > fordulo->fsz))
	{
	    item = NULL;
	}
    }
    
    if (item)
    {
	return (int)item->flag;
    }
    
    return (int)cache_item_flag_none;
}

extern int cache_check_spell(t_fordulo * fordulo, t_cache * cache, int ssz, long * count, long * curr)
{
    t_cache_spell * spell;
    
    if (!cache)
    {
	return -1;
    }

    if ((cache->spell_max_ssz < ssz) || (ssz < 1))
    {
	return -2;
    }
    
    spell = &cache->spells[ssz];
    if (spell)
    {
	if ((spell->ssz == ssz))
	{
	    if (count)
	    {
		*count = spell->count;
	    }
	    if (curr)
	    {
		*curr = spell->curr;
	    }
	    return (int)spell->flag;
	}
    }
    
    return (int)cache_spell_flag_none;
}

extern int cache_check_stat(t_fordulo * fordulo, t_cache * cache, int ssz, long * value)
{
    t_cache_stat * stat;
    
    if (!cache)
    {
	return -1;
    }
    
    if ((cache->stat_max_ssz < ssz) || (ssz < 0))
    {
	 return -2;
    }
    
    stat = &cache->stats[ssz];
    if (stat->ssz == ssz)
    {
	*value = stat->value;
	return 1;
    }
    return 0;
}

extern t_cache * cache_create(t_fordulo * fordulo)
{
    t_cache * cache;
    t_cache_monster * monsters;
    t_cache_item * items;
    t_cache_spell * spells;
    t_cache_stat * stats;
    int monster_max_ssz = 0;
    int item_max_ssz = 0;
    int spell_max_ssz = 0;
    int stat_max_ssz = 0;
    t_csata * csata;
    
    if (!fordulo || !fordulo->karakter)
    {
	return NULL;
    }
    if (!(monster_max_ssz = monsters_get_max()) || !(item_max_ssz = targy_get_max()) || !(spell_max_ssz = spell_get_max()) || !(stat_max_ssz = stat_get_max()))
    {
	return 0;
    }
    
    if (!(cache = malloc(sizeof(t_cache))))
    {
	return NULL;
    }
    
    if (!(monsters = (t_cache_monster *)malloc(sizeof(t_cache_monster) * (monster_max_ssz + 1))))
    {
	free(cache);
	return NULL;
    }
    if (!(items = (t_cache_item *)malloc(sizeof(t_cache_item) * (item_max_ssz + 1))))
    {
	free(cache);
	free(monsters);
	return NULL;
    }

    if (!(spells = (t_cache_spell *)malloc(sizeof(t_cache_spell) * (spell_max_ssz + 1))))
    {
	free(cache);
	free(monsters);
	free(items);
	return NULL;
    }

    if (!(stats = (t_cache_stat *)malloc(sizeof(t_cache_stat) * (stat_max_ssz + 1))))
    {
	free(cache);
	free(monsters);
	free(items);
	free(spells);
	return NULL;
    }

    memset(cache,0,sizeof(t_cache));
    memset(monsters,0,sizeof(t_cache_monster)*(monster_max_ssz + 1));
    memset(items,0,sizeof(t_cache_item)*(item_max_ssz + 1));
    memset(spells,0,sizeof(t_cache_spell)*(spell_max_ssz + 1));
    memset(stats,0,sizeof(t_cache_stat)*(stat_max_ssz + 1));

    cache->fsz = fordulo->fsz;
    cache->ksz = fordulo->karakter->ksz;
    cache->monster_max_ssz = monster_max_ssz;
    cache->item_max_ssz = item_max_ssz;
    cache->spell_max_ssz = spell_max_ssz;
    cache->stat_max_ssz = stat_max_ssz;
    cache->monsters = monsters;
    cache->items = items;
    cache->spells = spells;
    cache->stats = stats;
    
    return cache;
}

extern int cache_destroy(t_cache * cache)
{
    if (!cache)
    {
	return -1;
    }
    
    if (cache->monsters)
    {
	free(cache->monsters);
    }
    
    if (cache->items)
    {
	free(cache->items);
    }
    
    if (cache->spells)
    {
	free(cache->spells);
    }
    
    if (cache->stats)
    {
	free(cache->stats);
    }
    
    if (cache->monster_header)
    {
	if (cache->monster_header->fsz_arr)
	{
	    free(cache->monster_header->fsz_arr);
	}
	free(cache->monster_header);
    }

    if (cache->item_header)
    {
	if (cache->item_header->fsz_arr)
	{
	    free(cache->item_header->fsz_arr);
	}
	free(cache->item_header);
    }

    if (cache->spell_header)
    {
	if (cache->spell_header->fsz_arr)
	{
	    free(cache->spell_header->fsz_arr);
	}
	free(cache->spell_header);
    }

    if (cache->stat_header)
    {
	if (cache->stat_header->fsz_arr)
	{
	    free(cache->stat_header->fsz_arr);
	}
	free(cache->stat_header);
    }

    free(cache);
    
    return 0;
}

extern int cache_get_monster_max_ssz(t_cache * cache)
{
    if (cache)
    {
	return cache->monster_max_ssz;
    }
    return 0;
}

extern int cache_get_item_max_ssz(t_cache * cache)
{
    if (cache)
    {
	return cache->item_max_ssz;
    }
    return 0;
}

extern int cache_get_spell_max_ssz(t_cache * cache)
{
    if (cache)
    {
	return cache->spell_max_ssz;
    }
    return 0;
}

extern int cache_get_stat_max_ssz(t_cache * cache)
{
    if (cache)
    {
	return cache->stat_max_ssz;
    }
    return 0;
}

extern int cache_header_ref(t_cache * cache, t_cache_type type)
{
    t_cache_header * header;
    int ret = 0;
    int i;
    
    if (!cache)
    {
	return 0;
    }
    switch (type)
    {
	case cache_type_monster:
	    header = cache->monster_header;
	    break;
	case cache_type_item:
	    header = cache->item_header;
	    break;
	case cache_type_spell:
	    header = cache->spell_header;
	    break;
	case cache_type_stat:
	    header = cache->stat_header;
	    break;
	default:
	    header = NULL;
	    break;
    }
    
    if (header)
    {
	for (i = header->high_fsz; i >= 0; i--)
	{
	    if ((i%10 == 0) && ((header->fsz_arr[i] == cache_fsz_flag_saved) || (header->fsz_arr[i] == cache_fsz_flag_init)))
	    {
		ret = i;
		break;
	    }
	}
    }
    
    return ret;
}

extern int cache_check_monster_ismert(t_fordulo * fordulo, t_cache * cache)
{
    int ssz;
    int n = 0;
    
    if (!fordulo || !cache)
    {
	return 0;
    }
    
    for (n = 0, ssz = 1; ssz <= cache->monster_max_ssz; ssz++)
    {
	if ((cache->monsters[ssz].ssz != ssz) || (cache->monsters[ssz].fsz > fordulo->fsz))
	{
	    continue;
	}
	if (cache->monsters[ssz].flag == (int)cache_monster_flag_ismert)
	{
	    n++;
	}
    }
    return n;
}

extern int cache_check_item_ismert(t_fordulo * fordulo, t_cache * cache)
{
    int ssz;
    int n = 0;
    
    if (!fordulo || !cache)
    {
	return 0;
    }
    
    for (n = 0, ssz = 1; ssz <= cache->item_max_ssz; ssz++)
    {
	if ((cache->items[ssz].ssz != ssz) || (cache->items[ssz].fsz > fordulo->fsz))
	{
	    continue;
	}
	if ((cache->items[ssz].flag == (int)cache_item_flag_ismert) || (cache->items[ssz].flag == (int)cache_item_flag_hu))
	{
	    n++;
	}
    }
    return n;
}

extern int cache_dump(t_fordulo * fordulo, t_cache * cache)
{
    FILE * fp;
    char filename[127];
    t_cache_spell * spell;
    t_cache_item * item;
    t_cache_monster * monster;
    int i;
    int * ii;
    int ssz;
    
    if (!cachedebug)
    {
	return 0;
    }
    
    if (!fordulo || !cache)
    {
	return -1;
    }
    
    sprintf(filename, "cache.%d_%d.dump", fordulo->karakter->ksz, fordulo->fsz);
    if (!(fp = fopen(filename, "wb")))
    {
	return -1;
    }
    
    for (ssz = 1; ssz <= cache->monster_max_ssz; ssz++)
    {
	if (cache->monsters[ssz].ssz != ssz)
	{
	    continue;
	}
	
	ii = (int *)&cache->monsters[ssz];
	fprintf(fp, "monster_list: ssz=%4d ", cache->monsters[ssz].ssz);
	for (i = 0, ii++; i < (sizeof(t_cache_monster)-sizeof(long)-sizeof(long))/sizeof(int)-1; i++)
	{
	    fprintf(fp, "%4d, ", ii[i]);
	}
	fprintf(fp, "(%d/%d)", cache->monsters[ssz].count_megolt, cache->monsters[ssz].count_nemolt);
	fprintf(fp, "\n");
    }
    
    if (cache->monster_header)
    {
	fprintf(fp, "monster_header: high_fsz=%4d, cache_version=%4d\n", cache->monster_header->high_fsz, cache->monster_header->cache_version);
	if (cache->monster_header->fsz_arr)
	{
	    for (i = 0; i <= cache->monster_header->high_fsz; i++)
	    {
		fprintf(fp, "monster_header: i=%4d, fsz_arr[i]=%4d\n", i, cache->monster_header->fsz_arr[i]);
	    }
	}
	fprintf(fp, "\n");
    }
    
    for (ssz = 1; ssz <= cache->item_max_ssz; ssz++)
    {
	if (cache->items[ssz].ssz != ssz)
	{
	    continue;
	}
	
	ii = (int *)&cache->items[ssz];
	fprintf(fp, "item_list: ssz=%4d ", cache->items[ssz].ssz);
	for (i = 0, ii++; i < sizeof(t_cache_item)/sizeof(int)-1; i++)
	{
	    fprintf(fp, "%4d, ", ii[i]);
	}
	fprintf(fp, "\n");
    }
    
    if (cache->item_header)
    {
	fprintf(fp, "item_header: high_fsz=%4d, cache_version=%4d\n", cache->item_header->high_fsz, cache->item_header->cache_version);
	if (cache->item_header->fsz_arr)
	{
	    for (i = 0; i <= cache->item_header->high_fsz; i++)
	    {
		fprintf(fp, "item_header: i=%4d, fsz_arr[i]=%4d\n", i, cache->item_header->fsz_arr[i]);
	    }
	}
	fprintf(fp, "\n");
    }
    
    for (ssz = 1; ssz <= cache->spell_max_ssz; ssz++)
    {
	if (cache->spells[ssz].ssz != ssz)
	{
	    continue;
	}
	
	ii = (int *)&cache->spells[ssz];
	fprintf(fp, "spell_list: ssz=%4d ", cache->spells[ssz].ssz);
	for (i = 0, ii++; i < (sizeof(t_cache_spell)-sizeof(long))/sizeof(int)-1; i++)
	{
	    fprintf(fp, "%4d, ", ii[i]);
	}
	fprintf(fp, "%6ld", cache->spells[ssz].count);
	fprintf(fp, "\n");
    }
    
    if (cache->spell_header)
    {
	fprintf(fp, "spell_header: high_fsz=%4d, cache_version=%4d\n", cache->spell_header->high_fsz, cache->spell_header->cache_version);
	if (cache->spell_header->fsz_arr)
	{
	    for (i = 0; i <= cache->spell_header->high_fsz; i++)
	    {
		fprintf(fp, "spell_header: i=%4d, fsz_arr[i]=%4d\n", i, cache->spell_header->fsz_arr[i]);
	    }
	}
	fprintf(fp, "\n");
    }
    
    for (ssz = 1; ssz <= cache->stat_max_ssz; ssz++)
    {
	if (cache->stats[ssz].ssz != ssz)
	{
	    continue;
	}
	fprintf(fp, "stat_list: ssz=%4d", cache->stats[ssz].ssz);
	fprintf(fp, "%4d", cache->stats[ssz].fsz);
	fprintf(fp, "%14ld", cache->stats[ssz].value);
	fprintf(fp, "\n");
    }
    
    if (cache->stat_header)
    {
	fprintf(fp, "stat_header: high_fsz=%4d, cache_version=%4d\n", cache->stat_header->high_fsz, cache->stat_header->cache_version);
	if (cache->stat_header->fsz_arr)
	{
	    for (i = 0; i <= cache->stat_header->high_fsz; i++)
	    {
		fprintf(fp, "stat_header: i=%4d, fsz_arr[i]=%4d\n", i, cache->stat_header->fsz_arr[i]);
	    }
	}
	fprintf(fp, "\n");
    }
    
    fclose(fp);
    return 0;    
}

extern int do_cachedebug(char * progname)
{
    cachedebug = 1;
    return 0;
}

extern int do_cache(char * progname, char * nev, char * kar)
{
    int i;
    int ksz;
    t_cache_job * job;
    
    i = 0;
    ksz = 0;
    job = NULL;
    
    if (!nev || !kar)
    {
	return 1;
    }
    
    for (i = 0; i < sizeof(cache_work)/sizeof(t_cache_work); i++)
    {
	if (strncmp(nev, cache_work[i].nev, strlen(cache_work[i].nev)) == 0)
	{
	    if (isdigit2(*kar))
	    {
		ksz = atoi(kar);
		
		if (job = malloc(sizeof(t_cache_job)))
		{
		    memset(job, 0 , sizeof(t_cache_job));
		    job->ksz = ksz;
		    job->i = i;
		    
		    if (cache_job_last)
		    {
			cache_job_last->next = job;
		    }
		    if (!cache_job_first)
		    {
			cache_job_first = job;
		    }
		    cache_job_last = job;
		}
	    }
	}
    }
    
    return 0;
}

extern int cache_job_init(void)
{
    t_cache_job * job = NULL;
    t_cache_job * next = NULL;
    
    for (job = cache_job_first, next = NULL; job; job = next)
    {
	next = job->next;
	free(job);
    }
    cache_job_first = NULL;
    cache_job_last = NULL;
    
    return 0;
}

extern int cache_job_destroy(void)
{
    cache_job_init();
}

extern int cache_job_proc(void)
{
    t_cache_job * job;
    int ret = 0;
    
    for (job = cache_job_first; job; job = job->next)
    {
	if ((job->i >= 0) && (job->i < sizeof(cache_work)/sizeof(t_cache_work)))
	{
	    ret = (* cache_work[job->i].func) (job->ksz);
	}
    }
    
    return ret;
}

int kar_select(char * name, int ksz)
{
    int n;
    int m;
    int ret = 1;
    char buf[5];
    
    if (!name)
    {
	return 0;
    }
    
    sprintf(buf, "%4d", ksz);
    
    if (strncmp(name, buf, 4) !=0)
    {
	return 0;
    }
    
    for(n = 4, m = 0; n < strlen(name); n++)
    {
	if ((n - m < 4) && isdigit2(name[n]))
	{
	    continue;
	}
	if ((n == 4) && ((name[n] == '_') || (name[n] == 'w')))
	{
	    m = n;
	    continue;
	}
	if (m && (n - m < 5) && (name[n] == '.'))
	{
	    m = n;
	    continue;
	}
	if (m && (n - m == 1) && ((name[n] == 'h') || (name[n] == 'H')))
	{
	    continue;
	}
	if (m && (n - m == 2) && ((name[n] == 't') || (name[n] == 'T')))
	{
	    continue;
	}
	if (m && (n - m == 3) && ((name[n] == 'm') || (name[n] == 'M')))
	{
	    continue;
	}
	
	ret = 0;
	break;
    }
    
    return ret;
}

extern int cache_build_function(int ksz, int mode)
{
    char ** filelist;
    int i, j, k, n;
    t_forras * forras = NULL;
    t_forras_type forras_type = forras_type_none;
    t_fordulo * fordulo = NULL;
    int forduloszamok[10000];
    
    memset(forduloszamok, 0, sizeof(forduloszamok));
    k = 0;
    n = dir_list(bconf_get_indir(), "*", &filelist);

    if (n < 0)
    {
#ifdef DEBUG
	debug(NULL, "cache_build: fileliest error\n");
#endif
    }
    else if (filelist)
    {
	for(i = 0, j = 1; i < n; i++)
	{
	    if (filelist[i] && *filelist[i])
	    {
		if (!kar_select(filelist[i], ksz))
		{
		    free(filelist[i]);
		    filelist[i] = NULL;
		    continue;
		}
		forras_type = forras_type_file;
		if (forras = forras_create(filelist[i], forras_type))
		{
		    forras->n = j;
		    fordulo = fordulo_load_elozo(forras);
		}
		if (fordulo)
		{
		    if (forduloszamok[fordulo->fsz] == 0)
		    {
			j++;
			kiir(fordulo, "%3d. file: %s -> %d karakter %d fordulo: Cacheing... ",forras->n, forras->name, fordulo->karakter->ksz, fordulo->fsz);
			
			set_cache_mode(1);
			if ((forras->n > 1) || (mode == 1))
			{
			    cache_load(fordulo);
			}
			search_init(fordulo);
			if (reformat(fordulo) > 0)
			{
			    k++;
			    forduloszamok[fordulo->fsz] = 1;
			}
			cache_save(fordulo);
			cache_unload(fordulo);
			set_cache_mode(0);
			
			kiir(fordulo, "Done.\r\n");
		    }
		    else
		    {
			kiir(fordulo, "%3d. file: %s -> %d karakter %d fordulo: Mr cachelve van.\r\n",forras->n, forras->name, fordulo->karakter->ksz, fordulo->fsz);
		    }
		    fordulo_unload(fordulo);
		    fordulo = NULL;
		}
		forras_destroy(forras);
		forras = NULL;
		
		free(filelist[i]);
		filelist[i] = NULL;
	    }
        }
        free(filelist);
    }
    
    return k;
}

extern int cache_build(int ksz)
{
    return cache_build_function(ksz, 0);
}

extern int cache_update(int ksz)
{
    return cache_build_function(ksz, 1);
}

extern int cache_header_fordulok_kiir(t_fordulo * fordulo, t_cache * cache, t_cache_type type)
{
    t_cache_header * cache_header;
    int i;
    int tol, last;
    int high_fsz;
    int hh;
    
    if (!fordulo || ! cache)
    {
	return -1;
    }
    
    switch(type)
    {
	case cache_type_monster:
	    cache_header = cache->monster_header;
	    hh = 1;
	    break;
	case cache_type_item:
	    cache_header = cache->item_header;
	    hh = 1;
	    break;
	case cache_type_spell:
	    cache_header = cache->spell_header;
	    hh = 0;
	    break;
	case cache_type_stat:
	    cache_header = cache->stat_header;
	    hh = 0;
	    break;
	default:
	    cache_header = NULL;
	    high_fsz = 0;
	    break;
    }
    
    if (cache_header)
    {
	if (hh)
	{
	    high_fsz = (fordulo->fsz < cache_header->high_fsz ? fordulo->fsz : cache_header->high_fsz);
	}
	else
	{
	    high_fsz = cache_header->high_fsz;
	}
	for (i = 0, tol = -3, last = -2; (i <= high_fsz); i++)
	{
	    if ((cache_header->fsz_arr[i] == cache_fsz_flag_saved) || (cache_header->fsz_arr[i] == cache_fsz_flag_init))
	    {
		if ((last != i - 1) || (i == high_fsz))
		{
		    output(fordulo, "%s%s%d%s", (tol<0?"<i>(":""), (tol==last?((i==high_fsz)&&(last==i-1)?"-":", "):((i==high_fsz)&&(tol>=0)?"-":"")), i, (i==high_fsz?")</i>":""));
		    tol = i;
		}
		last = i;
	    }
	    else
	    {
		if (((last == i - 1) && (tol != i - 1)) || (last > 0 && (i == high_fsz)))
		{
		    output(fordulo, "%s%d%s", ((tol!=last)&&(tol>=0)?"-":""), last, (i==high_fsz?")</i>":""));
		    tol = last;
		}
	    }
	}
    }
}
