/*
 * Copyright (C) 2003-2005  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#include "typedef.h"
#ifndef INCLUDED_CACHE_TYPES
#define INCLUDED_CACHE_TYPES

#define CACHE_MONSTER_MEGOLES_LEN 5

typedef enum
{
    cache_type_none = 0,
    cache_type_monster = 1,
    cache_type_item = 2,
    cache_type_spell = 3,
    cache_type_stat = 4
} t_cache_type;

typedef enum
{
    cache_monster_flag_none = 0x00,
    cache_monster_flag_ismert = 0x01
} t_cache_monster_flag;

typedef enum
{
    cache_item_flag_none = 0x00,
    cache_item_flag_ismert = 0x01,
    cache_item_flag_hu = 0x02
} t_cache_item_flag;

typedef enum
{
    cache_spell_flag_none = 0x00,
    cache_spell_flag_ismert = 0x01,
} t_cache_spell_flag;

typedef enum
{
    cache_monster_megoli_flag_none = 0x00,
    cache_monster_megoli_flag_megoli = 0x01,
    cache_monster_megoli_flag_nemoli = 0x02
} t_cache_monster_megoli_flag;

#ifdef CACHE_INTERNAL_ACCESS
typedef enum
{
    cache_fsz_flag_none = 0,
    cache_fsz_flag_init = 1,
    cache_fsz_flag_saved = 2
} t_cache_fsz_flag;

typedef struct cache_monster_megoles
{
    int fsz __attribute__((__packed__));
    int flag __attribute__((__packed__));
} t_cache_monster_megoles __attribute__((__packed__));

typedef struct cache_monster
{
    int ssz __attribute__((__packed__));
    int fsz __attribute__((__packed__));
    int flag __attribute__((__packed__));
    t_cache_monster_megoles megoles[CACHE_MONSTER_MEGOLES_LEN]  __attribute__((__packed__));
    int count_megolt;
    int count_nemolt;
} t_cache_monster __attribute__((__packed__));

typedef struct cache_item
{
    int ssz __attribute__((__packed__));
    int fsz __attribute__((__packed__));
    int flag __attribute__((__packed__));
} t_cache_item __attribute__((__packed__));

typedef struct cache_spell
{
    int ssz __attribute__((__packed__));
    int fsz __attribute__((__packed__));
    int flag __attribute__((__packed__));
    long count __attribute__((__packed__));
    long curr __attribute__((__packed__));
} t_cache_spell __attribute__((__packed__));

typedef struct cache_stat
{
    int ssz __attribute__((__packed__));
    int fsz __attribute__((__packed__));
    long value __attribute__((__packed__));
} t_cache_stat __attribute__((__packed__));

typedef enum
{
    cache_version_none = 0x00,
    cache_version_1 = 0x01,
    cache_version_2 = 0x02,
    cache_version_3 = 0x03,
    cache_version_4 = 0x04    
} t_cache_version;

typedef struct cache_header
{
    t_cache_version cache_version;
    int high_fsz;
    char * fsz_arr;
} t_cache_header;

typedef struct cache_conf
{
    t_cache_type type;
    int magick;
    char * name;
    int paramcount;
    int (* cache_add_header)();
    int (* cache_add)();
    char * (* cache_get_header)();
    int (* cache_get)();
    int (* cache_get_max_ssz)();
} t_cache_conf;

typedef struct cache_work
{
    char * nev;
    int (* func)();
} t_cache_work;

typedef struct cache_job
{
    int ksz;
    int i;
    struct cache_job * next;
} t_cache_job;
#endif

typedef struct cache_data
{
    int ssz;
    int fsz;
    int flag;
} t_cache_data;

typedef struct cache
{
#ifdef CACHE_INTERNAL_ACCESS
    int ksz;
    int fsz;
    int monster_max_ssz;
    t_cache_header * monster_header;
    t_cache_monster * monsters;
    int item_max_ssz;
    t_cache_header * item_header;
    t_cache_item * items;
    int spell_max_ssz;
    t_cache_header * spell_header;
    t_cache_spell * spells;
    t_cache_spell * spells_curr;
    int stat_max_ssz;
    t_cache_header * stat_header;
    t_cache_stat * stats;
    t_cache_stat * stats_curr;
#endif
} t_cache;

#endif

#ifndef JUST_NEED_TYPES
#ifndef INCLUDED_CACHE_PROTOS
#define INCLUDED_CACHE_PROTOS

extern t_cache * cache_create(t_fordulo * fordulo);
extern int cache_destroy(t_cache * cache);
extern int cache_load(t_fordulo * fordulo);
extern int cache_save(t_fordulo * fordulo);
extern int cache_unload(t_fordulo * fordulo);
extern int cache_add_monster(t_cache * cache, int * buf, int n);
extern int cache_add_item(t_cache * cache, int * buf, int n);
extern int cache_add_spell(t_cache * cache, int * buf, int n);
extern int cache_add_stat(t_cache * cache, int * buf, int n);
extern int cache_add_monster_megoles(t_cache * cache, int ssz, t_cache_monster_megoli_flag flag);
extern int cache_add_spell_count(t_fordulo * fordulo, t_cache * cache, int ssz);
extern int cache_add_stat_value(t_cache * cache, int ssz, long value);
extern int cache_check_monster(t_fordulo * fordulo, t_cache * cache, int ssz, int * percent, int last_n, int * last_a);
extern int cache_check_item(t_fordulo * fordulo, t_cache * cache, int ssz);
extern int cache_check_spell(t_fordulo * fordulo, t_cache * cache, int ssz, long * count, long * curr);
extern int cache_check_stat(t_fordulo * fordulo, t_cache * cache, int ssz, long * value);
extern int cache_header_ref(t_cache * cache, t_cache_type type);
extern int cache_check_monster_ismert(t_fordulo * fordulo, t_cache * cache);
extern int cache_check_item_ismert(t_fordulo * fordulo, t_cache * cache);

#ifdef CACHE_INTERNAL_ACCESS
extern int cache_get_monster_max_ssz(t_cache * cache);
extern int cache_get_item_max_ssz(t_cache * cache);
extern int cache_get_spell_max_ssz(t_cache * cache);
extern int cache_get_stat_max_ssz(t_cache * cache);
extern int cache_get_monster(int ssz, t_cache * cache, int * result);
extern int cache_get_item(int ssz, t_cache * cache, int * result);
extern int cache_get_spell(int ssz, t_cache * cache, int * result);
extern int cache_get_stat(int ssz, t_cache * cache, int * result);
extern int cache_add_header_monster(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr);
extern int cache_add_header_item(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr);
extern int cache_add_header_spell(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr);
extern int cache_add_header_stat(t_cache * cache, t_cache_version cache_version, int high_fsz, char * fsz_arr);
extern char * cache_get_header_monster(t_cache * cache, int * buf, int * ver);
extern char * cache_get_header_item(t_cache * cache, int * buf, int * ver);
extern char * cache_get_header_spell(t_cache * cache, int * buf, int * ver);
extern char * cache_get_header_stat(t_cache * cache, int * buf, int * ver);
# ifdef DEBUG_CACHE
extern int cache_dump(t_fordulo * fordulo, t_cache * cache);
# endif
#endif

extern int do_cache(char * progname, char * nev, char * kar);
extern int do_cachedebug(char * progname);
extern int cache_job_init(void);
extern int cache_job_destroy(void);
extern int cache_job_proc(void);
extern int cache_build(int ksz);
extern int cache_update(int ksz);
#endif
#endif
