/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define CONF_INTERNAL_ACCESS
#include "include.h"
#include "conf.h"
#include "globals.h"
#include "output.h"
#include "targy.h"
#include "torony.h"

static int hibakiir = -1;
static t_aconf aconf;
static t_bconf bconf;

#define BCONF_HATAR 16

static t_conf_str bconf_def[] =
{
 {"conffile", CONF_FILE, confset_none},
 {"confdir", ".", confset_none|confset_commandline},
 {"karconfdir", KARCONF_DIR, confset_conf|confset_commandline},
 {"txtdir", TXT_DIR, confset_conf|confset_commandline},
 {"outdir", NULL, confset_conf|confset_kar|confset_commandline},
 {"indir", ".", confset_conf|confset_kar|confset_commandline},
 {"winul40dir", NULL, confset_conf|confset_commandline},
 {"notesdir", NULL, confset_conf|confset_commandline},
 {"cachedir", CACHE_DIR, confset_conf|confset_commandline},
 {"file_karakter", KARAKTER_TXT, confset_conf|confset_commandline},
 {"file_monster", MONSTER_TXT, confset_conf|confset_commandline},
 {"file_targy", TARGY_TXT, confset_conf|confset_commandline},
 {"file_varazslat", VARAZSLAT_TXT, confset_conf|confset_commandline},
 {"file_phua", PHUA_TXT, confset_conf|confset_commandline},
 {"file_titkosiras", TIM_TXT, confset_conf|confset_commandline},
 {"webhyper", WEBHYPER_HYPERLINK, confset_conf|confset_commandline},
 {"color_trezsor", COLOR_TREZSOR, confset_conf|confset_kar|confset_commandline},
 {"color_koordinata", COLOR_KOORDINATA, confset_conf|confset_kar|confset_commandline},
 {"color_onanalizis", COLOR_ONANALIZIS, confset_conf|confset_kar|confset_commandline},
 {"color_webhyperlink", COLOR_WEBHYPERLINK, confset_conf|confset_kar|confset_commandline},
 {"color_figyelt_keves", COLOR_FIGYELT_KEVES, confset_conf|confset_kar|confset_commandline},
 {"color_figyelt_sok", COLOR_FIGYELT_SOK, confset_conf|confset_kar|confset_commandline},
 {NULL, NULL, 0}
};

static t_conf_int aconf_def[] =
{
 { "reformat_ktjelentes", 1},
 { "winul_nextul", 0},
 { "kiemeles_mozgas", 1},
 { "kiemeles_karakterszam", 1},
 { "tudatkutatas_karszam", 1},
 { "kiemeles_szornyszam", 1},
 { "hyperlink_szorny", 1},
 { "kiemeles_epulet", 1},
 { "hyperlink_epulet", 1},
 { "hyperlink_targy", 1},
 { "hyperlink_spell", 1},
 { "kiszamolas_epulet_koordinata", 1},
 { "notes_labikoordinata", 1},
 { "kiemeles_szakicsillag", 1},
 { "kiemeles_szornykompi", 1},
 { "kiemeles_trezsor", 1},
 { "kiemeles_zarknod", 1},
 { "kiemeles_buvoles", 1},
 { "kiemeles_ef", 1},
 { "kiemeles_szornyehes", 1},
 { "kiemeles_varazslatlopas", 1},
 { "kiemeles_targypusztulas", 1},
 { "kiemeles_szopoko", 1},
 { "kiemeles_jellemvaltozas", 1},
 { "kiemeles_nincskompi", 1},
 { "kiemeles_onanalizis", 1},
 { "megfejtes_phua", 1},
 { "megfejtes_tim", 1},
 { "csata_stat_kiiras", 1},
 { "csata_sebzes", 1},
 { "csata_vampirizacio", 1},
 { "csata_nekrocsok", 1},
 { "csata_trafo", 1},
 { "csata_kontaktus", 1},
 { "csata_oregedes", 1},
 { "csata_papirang", 1},
 { "csata_morgan", 1},
 { "csata_kreitonfozet", 1},
 { "csata_ep_osszesen", 1},
 { "korvege_sebzes_stat", 1},
 { "korvege_sebzodes_stat", 1},
 { "korvege_csata_gyogyulas_stat", 1},
 { "korvege_csata_vp_stat", 1},
 { "korvege_csata_pp_stat", 1},
 { "korvege_csata_tu_stat", 1},
 { "korvege_csata_tvp_stat", 1},
 { "korvege_csata_oregedes_stat", 1},
 { "korvege_csata_papirang_stat", 1},
 { "korvege_eroforras_ep_stat", 1},
 { "korvege_eroforras_vp_stat", 1},
 { "korvege_eroforras_pp_stat", 1},
 { "korvege_eroforras_tu_stat", 1},
 { "korvege_eroforras_tvp_stat", 1},
 { "korvege_eroforras_ladiko_stat", 1},
 { "korvege_permanens_max_stat", 1},
 { "korvege_permanens_instabil_stat", 1},
 { "korvege_permanens_tulajdonsag_stat", 1},
 { "korvege_permanens_jellem_stat", 1},
 { "korvege_permanens_oregedes_stat", 1},
 { "korvege_permanens_papirang_stat", 1},
 { "korvege_spell_hasznalat_kiiras", 1},
 { "korvege_kalandozocsata_kiiras", 1},
 { "korvege_mezo_kiiras", 1},
 { "korvege_psziernyo_kiiras", 1},
 { "korvege_figyelmeztetes_kiiras", 1},
 { "notes_figyelmeztetes", 1},
 { "karlap_ismeretek_kiiras", 1},
 { "karlap_tno_osszeg_kiiras", 1},
 { "karlap_bonuszok_kiiras", 1},
 { "karlap_spell_erosseg_kiiras", 1},
 { "karlap_spell_oszto_kiiras", 1},
 { "karlap_vpregen_kiiras", 1},
 { "karlap_targy_bonusz_kiiras", 1},
 { "karlap_targy_kepesseg_kiiras", 1},
 { "karlap_targy_spell_kiiras", 1},
 { "karlap_spell_sebzes_kiiras", 1},
 { "spell_sebzes_spellero_bonusz", 1},
 { "spell_sebzes_magia_bonusz", 1},
 { "spell_sebzes_bonusz_csak_meglevo", 0},
 { "karlap_fegyver_sebzes_kiiras", 1},
 { "karlap_targy_tipusonkent_pancel", 0},
 { "karlap_targy_tipusonkent_ekszer", 1},
 { "karlap_targy_tipusonkent_kulcsok", 1},
 { "karlap_targy_tipusonkent_zarknod", 1},
 { "karlap_targy_tipusonkent_cirkusz", 1},
 { "karlap_targy_tipusonkent_ejfatty", 1},
 { "karlap_targy_tipusonkent_moa", 1},
 { "karlap_targy_tipusonkent_ryuku", 1},
 { "karlap_targy_ertek_kiiras", 1},
 { "karlap_targy_suly_kiiras", 1},
 { "karlap_targy_valtozas_kiiras", 1},
 { "karlap_kihivas_torony_kiiras", 1},
 { "karlap_kihivas_torony_reszletes", 1},
 { "fullstats_kiiras_stat", 1},
 { "fullstats_kiiras_varazslat", 1},
 { "notes_phua_jlk", 1},
 { "notes_tim_jlk", 1},
 { "targy_figyelt", 0},
 { "spell_figyelt", 0},
 { "szorny_oles", 0},
 { NULL, 0 }
};

extern int clearconf(void)
{
    memset(&aconf,0,sizeof(t_aconf));
    memset(&bconf,0,sizeof(t_bconf));
}

extern int aconf_get_curr(char * name)
{
    int i;
    t_aconf_line * aaconf_line;
    
    aaconf_line = (t_aconf_line *)&aconf;
    
    if (name != NULL)
    {
	for (i = 0; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
	{
	    if (strcmp(aconf_def[i].name, name) == 0)
	    {
#ifdef DEBUG
//	    	debug(NULL, "aconf_get_curr: directive=\"%s\", value=%d\n", aconf_def[i].name, aaconf_line[i].curr);
#endif
		return aaconf_line[i].curr;
	    }
	}
    }
    return -1;
}

extern char * bconf_get_value(char * name, t_conftype type)
{
    int i;
    t_bconf_line * bbconf_line;
    char * value;

    bbconf_line = (t_bconf_line *)&bconf;

    if (name != NULL)
    {
	for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
	{
	    if (strcmp(bconf_def[i].name, name) == 0)
	    {
		value = NULL;
		if (type == conftype_def)
		{
		    value = bbconf_line[i].def;
		}
		else if (type == conftype_global)
		{
		    value = bbconf_line[i].global;
		}
		else if (type == conftype_user)
		{
		    value = bbconf_line[i].user;
		}
		else if (type == conftype_commandline)
		{
		    value = bbconf_line[i].commandline;
		}
		else if (type == conftype_curr)
		{
		    value = bbconf_line[i].curr;
		}
#ifdef DEBUG
	    	debug(NULL, "bconf_get_value: directive=\"%s\", type=\"%d\", value=\"%s\"\n", bconf_def[i].name, type, value);
#endif
		return value;
	    }
	}
    }
    return NULL;
}

extern char * bconf_get_value_pri(char * name)
{
    int i;
    t_bconf_line * bbconf_line;
    char * value;

    bbconf_line = (t_bconf_line *)&bconf;

    if (name != NULL)
    {
	for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
	{
	    if (strcmp(bconf_def[i].name, name) == 0)
	    {
		value = NULL;
		if (bbconf_line[i].commandline)
		{
		    value = bbconf_line[i].commandline;
		}
		else if (bbconf_line[i].curr && (bconf_def[i].confset&confset_kar))
		{
		    value = bbconf_line[i].curr;
		}
		else if (bbconf_line[i].user)
		{
		    value = bbconf_line[i].user;
		}
		else if (bbconf_line[i].global)
		{
		    value = bbconf_line[i].global;
		}
		else
		{
		    value = bbconf_line[i].def;
		}
#ifdef DEBUG
	    	debug(NULL, "bconf_get_value_pri: directive=\"%s\", value=\"%s\"\n", bconf_def[i].name, value);
#endif
		return value;
	    }
	}
    }
    return NULL;
}

extern char * bconf_set_value(char * name, t_conftype type, char * value)
{
    int i;
    t_bconf_line * bbconf_line;
    char ** s;
    
    bbconf_line = (t_bconf_line *)&bconf;

    if (name != NULL)
    {
	for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
	{
	    if (strcmp(bconf_def[i].name, name) == 0)
	    {
		s = NULL;
		if (type == conftype_def)
		{
		    s = &bbconf_line[i].def;
		}
		else if (type == conftype_global)
		{
		    s = &bbconf_line[i].global;
		}
		else if (type == conftype_user)
		{
		    s = &bbconf_line[i].user;
		}
		else if (type == conftype_commandline)
		{
		    s = &bbconf_line[i].commandline;
		}
		else if (type == conftype_curr)
		{
		    s = &bbconf_line[i].curr;
		}
		
		if (s)
		{
#ifdef DEBUG
	    	    debug(NULL, "bconf_set_value: directive=\"%s\", type=\"%d\", value=\"%s\", *s=\"%s\"\n", bconf_def[i].name, type, value, *s);
#endif
		    if (*s)
		    {
			free(*s);
		    }
		    if (value)
		    {
			*s = strdup(value);
		    }
		    else
		    {
			*s = NULL;
		    }
		    return *s;
		}
	    }
	}
    }
    return NULL;
}

extern char * bconf_get_outdir(void)
{
    char * ret = NULL;
        
    if (bconf.outdir.commandline)
    {
	ret = bconf.outdir.commandline;
    }
    else if (bconf.outdir.curr)
    {
	ret = bconf.outdir.curr;
    }
    else if (bconf.outdir.user)
    {
	ret = bconf.outdir.user;
    }
    else if (bconf.outdir.global)
    {
	ret = bconf.outdir.global;
    }
    else
    {
	ret = bconf.outdir.def;
    }
    
    return ret;
}

extern char * bconf_get_indir(void)
{
    char * ret = NULL;
        
    if (bconf.indir.commandline)
    {
	ret = bconf.indir.commandline;
    }
    else if (bconf.indir.curr)
    {
	ret = bconf.indir.curr;
    }
    else if (bconf.indir.user)
    {
	ret = bconf.indir.user;
    }
    else if (bconf.indir.global)
    {
	ret = bconf.indir.global;
    }
    else
    {
	ret = bconf.indir.def;
    }
    
    return ret;
}

extern char * bconf_get_notesdir(void)
{
    char * ret = NULL;
        
    if (bconf.notesdir.commandline)
    {
	ret = bconf.notesdir.commandline;
    }
    else if (bconf.notesdir.curr)
    {
	ret = bconf.notesdir.curr;
    }
    else if (bconf.notesdir.user)
    {
	ret = bconf.notesdir.user;
    }
    else if (bconf.notesdir.global)
    {
	ret = bconf.notesdir.global;
    }
    else
    {
	ret = bconf.notesdir.def;
    }
    
    return ret;
}

extern char * bconf_get_winul40dir(void)
{
    char * ret = NULL;
        
    if (bconf.winul40dir.commandline)
    {
	ret = bconf.winul40dir.commandline;
    }
    else if (bconf.winul40dir.curr)
    {
	ret = bconf.winul40dir.curr;
    }
    else if (bconf.winul40dir.user)
    {
	ret = bconf.winul40dir.user;
    }
    else if (bconf.winul40dir.global)
    {
	ret = bconf.winul40dir.global;
    }
    else
    {
	ret = bconf.winul40dir.def;
    }
    
    return ret;
}

extern char * bconf_get_webhyper(void)
{
    char * ret = NULL;
        
    if (bconf.webhyper.commandline)
    {
	ret = bconf.webhyper.commandline;
    }
    else if (bconf.webhyper.curr)
    {
	ret = bconf.webhyper.curr;
    }
    else if (bconf.webhyper.user)
    {
	ret = bconf.webhyper.user;
    }
    else if (bconf.webhyper.global)
    {
	ret = bconf.webhyper.global;
    }
    else
    {
	ret = bconf.webhyper.def;
    }
    
    return ret;
}

extern char * bconf_get_cachedir(void)
{
    char * ret = NULL;
    
    if (bconf.cachedir.commandline)
    {
	ret = bconf.cachedir.commandline;
    }
    else if (bconf.cachedir.curr)
    {
	ret = bconf.cachedir.curr;
    }
    else if (bconf.cachedir.user)
    {
	ret = bconf.cachedir.user;
    }
    else if (bconf.cachedir.global)
    {
	ret = bconf.cachedir.global;
    }
    else
    {
	ret = bconf.cachedir.def;
    }
    
    return ret;
}

extern int aconf_get_webhyper_offline(void)
{
    char tmp[10];
    char * webhyper;
    
    memset(tmp, 0, sizeof(tmp));
    webhyper = bconf_get_webhyper();
    sprintf(tmp, "%.7s", webhyper);
    str_tolower2(tmp);
    if (strcmp(tmp, "http://") == 0)
    {
	return 0;
    }
    return 1;
}

extern char * bconf_get_color_figyelt_keves(void)
{
    char * ret = NULL;
        
    if (bconf.color_figyelt_keves.commandline)
    {
	ret = bconf.color_figyelt_keves.commandline;
    }
    else if (bconf.color_figyelt_keves.curr)
    {
	ret = bconf.color_figyelt_keves.curr;
    }
    else if (bconf.color_figyelt_keves.user)
    {
	ret = bconf.color_figyelt_keves.user;
    }
    else if (bconf.color_figyelt_keves.global)
    {
	ret = bconf.color_figyelt_keves.global;
    }
    else
    {
	ret = bconf.color_figyelt_keves.def;
    }
    
    return ret;
}

extern char * bconf_get_color_figyelt_sok(void)
{
    char * ret = NULL;
        
    if (bconf.color_figyelt_sok.commandline)
    {
	ret = bconf.color_figyelt_sok.commandline;
    }
    else if (bconf.color_figyelt_sok.curr)
    {
	ret = bconf.color_figyelt_sok.curr;
    }
    else if (bconf.color_figyelt_sok.user)
    {
	ret = bconf.color_figyelt_sok.user;
    }
    else if (bconf.color_figyelt_sok.global)
    {
	ret = bconf.color_figyelt_sok.global;
    }
    else
    {
	ret = bconf.color_figyelt_sok.def;
    }
    
    return ret;
}

extern int conf_set_hard_default(void)
{
    int i;
    int j;
    char ** s;
    t_aconf_line * aaconf_line;
    t_bconf_line * bbconf_line;
    
    aaconf_line = (t_aconf_line *)&aconf;
    for (i = 0; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
    {
	aaconf_line[i].def = aconf_def[i].enabled;
	aaconf_line[i].curr = aconf_def[i].enabled;
    }

    bbconf_line = (t_bconf_line *)&bconf;
    for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
    {
	s = (char **)&bbconf_line[i];
	for (j = 0; (j < sizeof(t_bconf_line)/sizeof(char **)); j++)
	{
	    if (s[j])
	    {
#ifdef DEBUG
		debug(NULL, "conf_set_hard_default: %s s[%d]=\"%s\"\n", bconf_def[i].name, j, s[j]);
#endif
		free(s[j]);
		s[j] = NULL;
	    }
	}

	if (bconf_def[i].value)
	{
	    bbconf_line[i].def = strdup(bconf_def[i].value);
	}
	else
	{
	    bbconf_line[i].def = NULL;
	}
    }
}

extern int conf_set_default(void)
{
    int i;
    t_aconf_line * aaconf_line;
    t_bconf_line * bbconf_line;
    
    aaconf_line = (t_aconf_line *)&aconf;
    for (i = 0; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
    {
        aaconf_line[i].curr = aaconf_line[i].def;
    }

    bbconf_line = (t_bconf_line *)&bconf;
    for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
    {
	if (bbconf_line[i].curr)
	{
	    free(bbconf_line[i].curr);
	}
	bbconf_line[i].curr = NULL;
    }
}

extern int bconf_set_record_from_curr(t_conftype type, t_confset confset)
{
    int i;
    t_bconf_line * bbconf_line;
    char ** value;
    
    bbconf_line = (t_bconf_line *)&bconf;
    for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
    {
	if (confset&bconf_def[i].confset && bbconf_line[i].curr)
	{
	    value = NULL;
	    if (type == conftype_def)
	    {
		value = &bbconf_line[i].def;
	    }
	    else if (type == conftype_global)
	    {
		value = &bbconf_line[i].global;
	    }
	    else if (type == conftype_user)
	    {
		value = &bbconf_line[i].user;
	    }
	    else if (type == conftype_commandline)
	    {
		value = &bbconf_line[i].commandline;
	    }
	    if (value)
	    {
#ifdef DEBUG
		debug(NULL, "bconf_set_record_from_curr: directive=\"%s\", type=\"%d\", oldvalue=\"%s\", newvalue=\"%s\"\n", bconf_def[i].name, type, *value, bbconf_line[i].curr);
#endif
		if (*value)
		{
		    free(*value);
		}
		*value = bbconf_line[i].curr;
    		bbconf_line[i].curr = NULL;
	    }
	}
    }
}

extern int conf_load_str(char * filename, char * direktive, char * value)
{
    int i;
    t_bconf_line * bbconf_line;
    char * p1;
    char * temp;
    int ret = -1;
    
    bbconf_line = (t_bconf_line *)&bconf;
    
    if (!value)
    {
	return -2;
    }
    value++;
    if (!(p1 = strstr(value, "\"")))
    {
	return -2;
    }
    *p1 = '\0';

#ifdef DEBUG
    debug(NULL, "conf_load_str: file=\"%s\", directive=\"%s\", value=\"%s\"\n", filename, direktive, value);
#endif
    for (i = 0, temp = NULL; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
    {
	if (strcmp(bconf_def[i].name, direktive) == 0)
	{
	    if (strlen(value) <= 1)
	    {
		if (bbconf_line[i].curr)
		{
		    free(bbconf_line[i].curr);
		}
		bbconf_line[i].curr = NULL;
		ret = 1;
	    }
	    else if (temp = strdup(value))
	    {
		if (bbconf_line[i].curr)
		{
		    free(bbconf_line[i].curr);
		}
		bbconf_line[i].curr = temp;
		bbconf_line[i].read = 1;
#ifdef DEBUG
		debug(NULL, "conf_load_str: file=\"%s\", i=%d, directive=\"%s\", value=\"%s\"\n", filename, i, bconf_def[i].name, bbconf_line[i].curr);
#endif
		ret = 1;
	    }
	    else
	    {
		ret = -3;
	    }
	    break;
	}
    }
    
    return ret;
}

extern int conf_load_value(char * filename, char * l1, char * p1, t_confset confset, t_karakter * karakter)
{
    int r;
    int i;
    char * p2;
    char * p3;
    int value;
    t_aconf_line * aaconf_line;

    aaconf_line = (t_aconf_line *)&aconf;
    r = 1;
    
    value = atoi(p1);
    if (value && ((confset == confset_kar) || (confset == confset_commandline)) && (strcmp(l1, "szorny_oles") == 0))
    {

        int percent;
	
	if ((p1 = strstr(p1, "[")) && (p2 = strstr(p1, "]")))
	{
	    p1++;
	    p1 = skip_space(p1);
	    percent = atoi(p1);
	    for( ; (isdigit2(*p1) || isspace2(*p1)) && (p1 < p2); p1++);
	    if ((*p1 == ']') && (percent >= 0) && (percent <=100))
	    {
		if (torony_szorny_oles_set_percent(value, percent, confset) < 0)
		{
		    r = -3;
		}
		else
		{
		    aconf.szorny_oles.curr = 1;
		}
	    }
	}
    }
    else if (value && (strcmp(l1, "targy_figyelt") == 0))
    {
	int min;
	int max;
	
	min = 0;
	max = 0;
	p1 = skip_nonspace(p1);
	p1 = skip_space(p1);
	
	if ((*p1 == '[') && (p2 = strstr(p1, "]")))
	{
	    p1++;
	    p1 = skip_space(p1);
	    min = atoi(p1);
	    for( ; (isdigit2(*p1) || isspace2(*p1)) && (p1 < p2); p1++);
	    if (*p1 == ',')
	    {
		for(p1++; !isdigit2(*p1) && (p1 < p2); p1++);
		max = atoi(p1);
		for( ; (isdigit2(*p1) || isspace2(*p1)) && (p1 < p2); p1++);
	    }
	    if (*p1 != ']')
	    {
		min = 0;
		max = 0;
	    }
	}
	if (targy_figyelt_add(karakter, value, confset, min, max) < 0)
	{
	    r = -3;
	}
	else
	{
	    aconf.targy_figyelt.curr = 1;
	}
    }
    else if (value && (strcmp(l1, "spell_figyelt") == 0))
    {
	int irany;
	
	for ( ; isdigit2(*p1); p1++);
	p1 = skip_space(p1);
	
	irany = 1;
	if (*p1 == '+')
	{
	    irany = 2;
	}
	
	if (spell_figyelt_add(value, confset, irany) < 0)
	{
	    r = -3;
	}
	else
	{
	    aconf.spell_figyelt.curr = 1;
	}
    }
    else if ((value != 1) && !((value == 0) && (*p1 == '0')) && !((value == 2) && (strcmp(l1, "spell_sebzes_bonusz_csak_meglevo") == 0)))
    {
	r = -2;
    }
    else
    {
	for (i = 0, r = -1; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
	{
	    if (strcmp(aconf_def[i].name, l1) == 0)
	    {
		r = 1;
		if (aaconf_line[i].commandline != 1)
		{
		    aaconf_line[i].curr = value;
		    aaconf_line[i].read = 1;
		    if (confset == confset_commandline)
		    {
			aaconf_line[i].commandline = 1;
			aaconf_line[i].def = value;
		    }
#ifdef DEBUG
		    debug(NULL, "conf_load: file=\"%s\", directive=\"%s\", value=%d\n", filename, aconf_def[i].name, aaconf_line[i].curr);
#endif
		}
		break;
	    }
	}
    }
    
    return r;
}

extern int conf_load(t_karakter * karakter, char * filename, t_confset confset)
{
    int i;
    FILE * fp;
    int currline;
    char line[LINE_MAXLEN];
    char * l1;
    char * p1;
    int r;
    int ret = 0;
    
#ifdef DEBUG
    debug(NULL, "conf_load: karakter=%p file=\"%s\"\n", karakter, filename);
#endif
    if (fp = fopen(filename, "rb"))
    {
	ret = 1;
	for (currline = 1; fgets(line, LINE_MAXLEN - 1, fp); currline++)
	{
	    l1 = skip_space(line);
	    if (!l1 || (*l1 == '#') || (*l1 == '\r') || (*l1 == '\n') || (*l1 == '\0'))
	    {
		continue;
	    }
	    
	    p1 = skip_nonspace(l1);
	    *p1 = '\0';
	    p1++;
	    p1 = skip_space(p1);
	     
	    if (*p1 != '=')
	    {
		kiir(NULL, "%sHiba a %s conffile %d. soraban (nincs a direktiva melle \'=\' jel)\r\n", (hibakiir ? "" : "\r\n"), filename, currline);
		hibakiir = 1;
		continue;
	    }
	    
	    p1++;
	    p1 = skip_space(p1);
	    
	    r = 1;
	    if (*p1 == '#')
	    {
		p1++;
	    }
	    if (!isdigit2(*p1))
	    {
		r = conf_load_str(filename, l1, p1);
	    }
	    else
	    {
		r = conf_load_value(filename, l1, p1, confset, karakter);
	    }
	    if (r == -3)
	    {
	        kiir(NULL, "%sERROR: %s conffile %d. soranak beolvasasa kozben memoria lefoglalasi hiba lepett fel\r\n", (hibakiir ? "" : "\r\n"), filename, currline);
	        exit(-1);
	    }
	    if (r == -2)
	    {
	        kiir(NULL, "%sFIGYELMEZTETES: Hiba a %s conffile %d. soraban (a beallitas nem 0/1 vagy \"szoveg\")\r\n", (hibakiir ? "" : "\r\n"), filename, currline);
	        hibakiir = 1;
	    }
	    else if (r == -1)
	    {
	        kiir(NULL, "%sFIGYELMEZTETES: Hiba a %s conffile %d. soraban \"%s\" (nincs ilyen beallitasi opcio)\r\n", (hibakiir ? "" : "\r\n"), filename, currline, l1);
	        hibakiir = 1;
	    }
	    else if (r == 0)
	    {
	        kiir(NULL, "%sFIGYELMEZTETES: Hiba a %s conffile %d. soraban (a direktiva itt nem allithato)\r\n", (hibakiir ? "" : "\r\n"), filename, currline);
	        hibakiir = 1;
	    }
	}
	fclose(fp);
    }
    return ret;
}

extern int conf_load_default(void)
{
    int i;
    int ok;
    t_conftype t[] = {conftype_def, conftype_global, conftype_user, conftype_commandline};
    char * filename;
    t_aconf_line * aaconf_line;
    
    aaconf_line = (t_aconf_line *)&aconf;

    hibakiir = -1;
    spell_figyelt_del_confset(confset_conf);
    targy_figyelt_del_confset(confset_conf);
    aconf.targy_figyelt.read = 1;
    aconf.spell_figyelt.read = 1;
    aconf.szorny_oles.read = 1;
    
    ok = 0;
//    if (!get_pdirconf())
    {
	for (i = 0; t[i] && (i < sizeof(t)/sizeof(t_conftype)) && !(get_pdirconf() && i); i++)
	{
	    if (filename = bconf_get_value("conffile", t[i]))
	    {
		if (conf_load(NULL, filename, confset_conf))
		{
		    ok++;
		    kiir(NULL, "Beolvasott conffile: %d. \"%s\"\r\n", ok, filename);
		    bconf_set_record_from_curr(t[i], confset_conf);
		}
		else
		{
		    bconf_set_value("conffile", t[i], NULL);
		}
	    }
	}
    }
/*    
    if (!ok)
    {
	if (filename = bconf_get_value("conffile", conftype_def))
	{
	    if (conf_load(NULL, filename, confset_conf))
	    {
		ok = 1;
		h_filename = filename;
		bconf_set_record_from_curr(conftype_def, confset_conf);
	    }
	}
    }
*/    
    if (ok)
    {
	for (i = 0; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
	{
	    aaconf_line[i].def = aaconf_line[i].curr;
	}
	kiir(NULL, "\r\n");
    }
    else
    {
	kiir(NULL, "Hasznalt conffile: nem sekerult egyet se beolvasni, a hardcoded beallitasok lesznek hasznalva.\r\n\r\n");
    }
}

extern int conf_load_karakter(t_karakter * karakter)
{
    int ksz;
    int i;
    int ok;
    t_conftype t[] = {conftype_commandline, conftype_user, conftype_global, conftype_def};
    char * filename;
    char * file;
    char * dir;
    char * outdir;
    char * indir;
    t_aconf_line * aaconf_line;
    
    conf_set_default();
    
    if (!karakter)
    {
	return -1;
    }
    
    ksz = karakter->ksz;

    if (ksz < 1000 || ksz > 9099)
    {
	return -1;
    }
    
    aaconf_line = (t_aconf_line *)&aconf;
    
    hibakiir = 0;
    spell_figyelt_del_confset(confset_kar);
    targy_figyelt_del_confset(confset_kar);
    torony_szorny_oles_lista_destroy(0);
    
    for (i = 0, ok = 0; !ok && t[i] && (i < sizeof(t)/sizeof(t_conftype)); i++)
    {
	if (dir = bconf_get_value("karconfdir", t[i]))
	{
	    if (filename = malloc(8 + 1 + 1))
	    {
		sprintf(filename, "%d.cfg", ksz);
		if (file = get_filename(dir, filename))
		{
		    if (conf_load(karakter, file, confset_kar))
		    {
			ok = 1;
		    }
		    free(file);
		}
		free(filename);
	    }
	}
    }
}

extern int init_conf (void)
{
    int i, j, k;
    char * file;
    char * def_conf_file;
    char * dir;
    char * p;
    int ok;
    char * s[] = {"karconfdir", "txtdir"};
    t_conftype t[] = {conftype_user, conftype_global, conftype_def};
    int kell;
    struct stat sfile;
    mode_t mode;
    FILE * fp1, * fp2;
    unsigned long size;
    char * cfgstr;

#ifdef DEBUG
    debug(NULL, "init_conf: 0\n");
#endif    

    if (get_pdirconf() && (dir = get_pdir()))
    {
	bconf_set_value("confdir", conftype_def, dir);
    }

    for (i = (get_pdirconf()?2:0); t[i] && (i < sizeof(t)/sizeof(t_conftype)); i++)
    {
	for (k = i, file = NULL; !file && (k < sizeof(t)/sizeof(t_conftype)); k++)
	{
	    file = bconf_get_value("conffile", t[k]);
	}
	dir = bconf_get_value("confdir", t[i]);
	
	if (p = get_filename(dir, file))
	{
	    bconf_set_value("conffile", t[i], p);
	    free(p);
	}
    }

#ifdef DEBUG
    debug(NULL, "init_conf: 1\n");
#endif    

    if (!get_pdirconf())
    {
	p = bconf_get_value("confdir", conftype_user);
	if (p && (stat(p, &sfile) != 0))
	{
#ifndef WIN32
	    mode = S_IRWXU;
	    if (mkdir(p, mode) != 0)
#else
	    if (mkdir(p) != 0)
#endif
	    {
		bconf_set_value("confdir", conftype_user, NULL);
	    }
	}
    
#ifdef DEBUG
	debug(NULL, "init_conf: 2\n");
#endif    

	def_conf_file = NULL;
	p = bconf_get_value("conffile", conftype_user);
	if (p && (stat(p, &sfile) != 0))
	{
	    if (!(file = bconf_get_value("conffile", conftype_global)))
	    {
		file = bconf_get_value("conffile", conftype_def);
	    }
	    else if (stat(file, &sfile) != 0)
	    {
		file = bconf_get_value("conffile", conftype_def);
	    }
	    if (file && (stat(file, &sfile) == 0))
	    {
		size = (unsigned long)sfile.st_size;
		if (cfgstr = malloc(size+1))
		{
		    memset(cfgstr,0,size+1);
		    if ((fp1 = fopen(file,"rb")) && (fp2 = fopen(p,"wb")))
		    {
			if(fread(cfgstr,1,size,fp1))
			{
			    fwrite(cfgstr,1,size,fp2);
			    fclose(fp1);
			    fclose(fp2);
#ifndef WIN32
			    mode = S_IRUSR|S_IWUSR;
			    chmod(p, mode);
#endif
			    def_conf_file = strdup(file);
			}
		    }
		    free(cfgstr);
		}
	    }
	}
    }

#ifdef DEBUG
    debug(NULL, "init_conf: 3\n");
#endif    

    conf_load_default();

#ifdef DEBUG
    debug(NULL, "init_conf: 4\n");
#endif    
    
    for (i = (get_pdirconf()?2:0); t[i] && (i < sizeof(t)/sizeof(t_conftype)); i++)
    {

	for (j = 0, ok = 0; s[j] && (j < sizeof(s)/sizeof(char *)); j++)
	{
	    for (k = i, file = NULL; !file && (k < sizeof(t)/sizeof(t_conftype)); k++)
	    {
		file = bconf_get_value(s[j], t[k]);
	    }
	    dir = bconf_get_value("confdir", t[i]);
	    
	    if (p = get_filename(dir, file))
	    {
#ifdef DEBUG
		debug(NULL, "init_conf: directive=\"%s\", type=\"%d\", value=\"%s\"\n", s[j], t[i], p);
#endif
		if ((t[i] == conftype_user) && ((j == 0)))
		{
		    if (p && (stat(p, &sfile) == 0))
		    {
			if (S_ISDIR(sfile.st_mode))
			{
			    ok = 1;
			}
		    }
		    else
		    {
#ifndef WIN32
			mode = S_IRWXU;
			if (mkdir(p, mode) == 0)
#else
			if (mkdir(p) == 0)
#endif
			{
			    ok = 1;
			}
		    }
		    
		}
	        else
		{
		    ok = 1;
		}
		
		if (ok)
		{
		    bconf_set_value(s[j], t[i], p);
		    free(p);
		    continue;
		}
		else
		{
		    bconf_set_value(s[j], t[i], NULL);
		}
		free(p);
	    }
	    else
	    {
		bconf_set_value(s[j], t[i], NULL);
	    }
	}
    }

#ifdef DEBUG
    debug(NULL, "init_conf: 5\n");
#endif    

    if (dir = bconf_get_value_pri("winul40dir"))
    {
	if (p = get_filename(dir, "notesz"))
	{
	    bconf_set_value("notesdir", conftype_user, p);
	    free(p);
	}
    }

    // cache dir
    p = bconf_get_cachedir();
    if (p && (stat(p, &sfile) != 0))
    {
#ifndef WIN32
	mode = S_IRWXU;
	if (mkdir(p, mode) != 0)
#else
	if (mkdir(p) != 0)
#endif
	{
	    for (i = 0; t[i] && (i < sizeof(t)/sizeof(t_conftype)); i++)
	    {
		bconf_set_value("cachedir", t[i], NULL);
	    }
	}
    }
    
//    if (!get_pdirconf())
    {
#ifdef DEBUG
	debug(NULL, "init_conf: 6\n");
#endif    

	{
	    t_bconf_line * bbconf_line;
	    t_aconf_line * aaconf_line;
	    
	    j = 0;
	    bbconf_line = (t_bconf_line *)&bconf;
	    for (i = BCONF_HATAR; (i < sizeof(bconf)/sizeof(t_bconf_line)) && bconf_def[i].name; i++)
	    {
		if (!bbconf_line[i].read && bbconf_line[i].def)
		{
		    if (!j)
		    {
			kiir(NULL, "# Az alabbi beallitasok nem szerepelnek a conf filekben, de leteznek.\r\n");
			kiir(NULL, "# Elofordulhat, hogy regi a conf filed, legyszi csekkold le, es ha kell frissitsd!\r\n");
			kiir(NULL, "# %s\r\n", REFORMAT_HYPERLINK);
		    }
		    kiir(NULL, "%s (default: \"%s\")\r\n", bconf_def[i].name, bbconf_line[i].def);
		    j++;
		}
	    }
	
	    aaconf_line = (t_aconf_line *)&aconf;
	    for (i = 0; aconf_def[i].name && (i < sizeof(aconf)/sizeof(t_aconf_line)); i++)
	    {
		if (!aaconf_line[i].read)
		{
		    if (!j)
		    {
			kiir(NULL, "# Az alabbi beallitasok nem szerepelnek a conf filekben, de leteznek.\r\n");
			kiir(NULL, "# Elofordulhat, hogy regi a conf filed, legyszi csekkold le, es ha kell frissitsd!\r\n");
			kiir(NULL, "# %s\r\n", REFORMAT_HYPERLINK);
		    }
		    kiir(NULL, "%s (default: %d)\r\n", aconf_def[i].name, aaconf_line[i].def);
		    j++;
		}
	    }
	    if (j)
	    {
		kiir(NULL, "\r\n");
	    }
	}
    }
    
#ifdef DEBUG
    debug(NULL, "init_conf: 7\n");
#endif    

}

extern int conf_unload (void)
{
    int i;
    int j;
    char ** s;
    t_conftype type[] = {conftype_def, conftype_global, conftype_user, conftype_commandline, conftype_curr};
    t_bconf_line * bbconf_line;
    
    bbconf_line = (t_bconf_line *)&bconf;
    for (i = 0; bconf_def[i].name && (i < sizeof(bconf)/sizeof(t_bconf_line)); i++)
    {
	s = (char **)&bbconf_line[i];
	for (j = 0; (j < (sizeof(t_bconf_line)/sizeof(char **) - 1)); j++)
	{
#ifdef DEBUG
	    debug(NULL, "conf_unload: directive=\"%s\", type=\"%d\", value=\"%s\"\n", bconf_def[i].name, type[j], s[j]);
#endif
	    if (s[j])
	    {
		free(s[j]);
		s[j] = NULL;
	    }
	}
    }
    spell_figyelt_destroy();
    targy_figyelt_destroy();
}
