/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define FEGYVER_INTERNAL_ACCESS
#include "include.h"
#include "fegyver.h"
#include "spell.h"
#include "karakter.h"
#include "sebzes.h"
#include "targy.h"

static char * fegyvertipus_neve[] =
{
    "Vziharc",
    "klvvs",
    "Szrfegyver",
    "Vgfegyver",
    "tfegyver",
    "Lfegyver",
    "Dobfegyver",
    NULL
};


extern char * get_fegyvertipus_neve(int i)
{
    if (i < (sizeof(fegyvertipus_neve)/sizeof(char*)))
    {
	return fegyvertipus_neve[i];
    }
    return NULL;
}

extern t_fegyver * fegyver_create(t_targy * targy, t_fegyver_type type, t_sebzes * sebzes, int spellssz, int spellero, int spellesely)
{
    t_fegyver * temp;
    t_spell * spell;
    char * s;
    
    if (!(temp = malloc(sizeof(t_fegyver))))
    {
#ifdef DEBUG
	debug(NULL, "fegyver_create: could not allocate memory for temp\n");
#endif
	return NULL;
    }
    
    memset(temp,0,sizeof(t_fegyver));
    
    temp->targy = targy;
    if (sebzes)
    {
	temp->sebzes.darab = sebzes->darab;
	temp->sebzes.ertek = sebzes->ertek;
	temp->sebzes.bonusz = sebzes->bonusz;
    }
    temp->type = type;
    temp->spellero = spellero;
    temp->spellesely = spellesely;
    temp->next = NULL;
    if (spellssz)
    {
	if (spell = get_spell_from_ssz(spellssz))
	{
	    temp->spell = spell;
	}
	else
	{
	    temp->spell = NULL;
	}
    }
    
#ifdef DEBUG
    debug(NULL, "fegyver_create: %s (#%d) type=%d, sebzes=%dD%d%+d, spell=%d\n", temp->targy->nev, temp->targy->ssz, temp->type, temp->sebzes.darab, temp->sebzes.ertek, temp->sebzes.bonusz, (temp->spell ? temp->spell->ssz : 0));
#endif
    return temp;
}

extern float get_okol_sebzes(t_karakter * karakter, t_fegyver * fegyver)
{
    int sebzes;
    int n;
    int hm;
    float ret;
    
    if (!karakter || !fegyver)
    {
	return 0;
    }
    
    if (karakter->kepessegek.harcmuveszet < 1)
    {
	sebzes = 0;
    }
    else if (karakter->kepessegek.harcmuveszet < 10)
    {
	sebzes = (karakter->kepessegek.harcmuveszet + 2) / 3;
    }
    else 
    {
	n = (karakter->kepessegek.harcmuveszet - 10) / 5;
	sebzes = 10 + n;
    }
    if ((karakter->faj == 7) && (karakter->szint >=9) && (fegyver->targy->ssz == 28))
    {
	sebzes += 9;
    }
    else
    {
	sebzes += 2;
    }
    
    ret = (float)sebzes;
    if (fegyver->targy->ssz != 28)
    {
        ret += sebzes_calculate(fegyver->sebzes);
    }

    return ret;
}

extern t_fegyver_type get_fegyver_type_from_targy_tipus(int ssz, t_targy_tipus tipus, int balkez)
{
    t_fegyver_type type;
    
    type = fegyver_type_none;

    if (ssz == 28)
    {
	type = fegyver_type_okol;
    }
    else if (tipus == targy_tipus_szuro)
    {
	type = fegyver_type_szuro;
    }
    else if (tipus == targy_tipus_vago)
    {
	type = fegyver_type_vago;
    }
    else if (tipus == targy_tipus_uto)
    {
	type = fegyver_type_uto;
    }
    else if (tipus == targy_tipus_dobo)
    {
	type = fegyver_type_dobo;
    }
    else if (tipus == targy_tipus_vizi)
    {
	type = fegyver_type_vizi;
    }
    else if (tipus == targy_tipus_pajzs)
    {
	if (ssz == 740)
	{
	    type = fegyver_type_uto|fegyver_type_balkez;
	}
	else if ((ssz == 630) || (ssz == 425))
	{
	    type = fegyver_type_vago|fegyver_type_balkez;
	}
    }
    else if (tipus == targy_tipus_kesztyu)
    {
	type = fegyver_type_okol;
    }
    else if (tipus == targy_tipus_lofegyver_lovedek)
    {
	type = fegyver_type_lo;
    }
    
    if (balkez)
    {
	type |= fegyver_type_balkez;
    }
    
    return type;
}
