/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define INIT_INTERNAL_ACCESS
#include "include.h"
#include "init.h"
#include "conf.h"
#include "sorszam.h"
#include "spell.h"
#include "fegyver.h"
#include "targy.h"
#include "phua.h"
#include "tit.h"
#ifdef WIN32
# include "win.h"
#endif

#ifndef CONF_DIR
# define CONF_DIR NULL
#endif


static t_initfiles initfiles[]=
{
 {"file_karakter", NULL, init_karakters, 1, 0},
 {"file_monster", NULL, init_monsters, 1, 0},
 {"file_varazslat", NULL, init_spell, 1, 0},
 {"file_targy", NULL, init_targy, 1, 0},
 {"file_phua", NULL, init_phua, 0, 0},
 {"file_titkosiras", NULL, init_tim, 0, 0},
 {NULL, NULL, NULL, 0, 0}
};

extern int init_txtfiles(void)
{
    char * dir;
    char * file;
    char * filename;
    t_conftype t[] = {conftype_commandline, conftype_user, conftype_global, conftype_def};
    char * p;
    int ok;
    int i, j, k;
    FILE *fp;
    
    for (j = 0, ok = 0; initfiles[j].name && (j < sizeof(initfiles)/sizeof(t_initfiles)); j++, ok = 0)
    {
	for (i = 0; !ok && t[i] && (i < sizeof(t)/sizeof(t_conftype)); i++)
	{
	    for (k = i, file = NULL; !file && (k < sizeof(t)/sizeof(t_conftype)); k++)
	    {
		file = bconf_get_value(initfiles[j].name, t[k]);
	    }
	    if (!(dir = bconf_get_value("txtdir", t[i])))
	    {
		continue;
	    }
	    if (!file)
	    {
		continue;
	    }
	    if (p = get_filename(dir, file))
	    {
		bconf_set_value(initfiles[j].name, t[i], p);
		filename = strdup(p);
		free(p);
	    }
	    else
	    {
		filename = strdup(file);
	    }
	    if (!ok)
	    {
		if (fp = fopen(filename, "rb"))
		{
		    ok = 1;
		    initfiles[j].file = strdup(filename);
		    fclose(fp);
#ifdef DEBUG
		    debug(NULL, "init_txtfiles: dir=\"%s\", name=\"%s\", file=\"%s\"\n", dir, initfiles[j].name, initfiles[j].file);
#endif
		}
	    }
	    
	    if (filename)
	    {
		free(filename);
		filename = NULL;
	    }
	}
    }
    
    for (i = 0; initfiles[i].name && (i < sizeof(initfiles)/sizeof(t_initfiles)); i++)
    {
#ifdef DEBUG
	debug(NULL, "init_txtfiles: try i=%d, name=\"%s\", file=\"%s\", autoload=%d\n", i, initfiles[i].name, initfiles[i].file, initfiles[i].autoload);
#endif
	if (initfiles[i].autoload && !initfiles[i].loaded && initfiles[i].file)
	{
	    initfiles[i].loaded = (*initfiles[i].func) (initfiles[i].file);
	}
    }
}

extern int load_txtfiles(char * name)
{
    int i;
    
    for (i = 0; initfiles[i].name && (i < sizeof(initfiles)/sizeof(t_initfiles)); i++)
    {
	if (strcmp(name, initfiles[i].name) == 0)
	{
#ifdef DEBUG
	    debug(NULL, "load_txtfiles: try i=%d, file=\"%s\", autoload=%d\n", i, initfiles[i].file, initfiles[i].autoload);
#endif
	    if (!initfiles[i].loaded && initfiles[i].file)
	    {
		initfiles[i].loaded = (*initfiles[i].func) (initfiles[i].file);
	    }
	}
    }
}

extern int deinit_txtfiles(void)
{
    int i;
    
    karakters_str_unset();
    monsters_str_unset();
    targy_list_destroy();
    spell_list_destroy();
    phua_list_destroy();
    tim_list_destroy();

    for (i = 0; initfiles[i].name && (i < sizeof(initfiles)/sizeof(t_initfiles)); i++)
    {
#ifdef DEBUG
	debug(NULL, "deinit_txtfiles: free i=%d, file=\"%s\"\n", i, initfiles[i].file);
#endif
	if (initfiles[i].file)
	{
	    free(initfiles[i].file);
	}
	initfiles[i].file = NULL;
    }
}

extern int init_program(void)
{
    char * home;
    char * dir;
    char * user_dir;
    char * p;
    
    clearconf();
    conf_set_hard_default();

#ifdef WIN32
    init_win();
#else
    bconf_set_value("confdir", conftype_global, CONF_DIR);
    if ((home = getenv("HOME")) && (strlen(home) > 1))
    {
	if (p = get_filename(home, USERCONF_DIR))
	{
	    bconf_set_value("confdir", conftype_user, p);
	    free(p);
	    p = NULL;
	}
    }
#endif
}

extern int deinit_program(void)
{
}
