/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#include "include.h"
#include "fordulo.h"
#include "format.h"
#include "output.h"
#include "globals.h"
#include "init.h"
#include "input.h"
#include "conf.h"
#include "commandline.h"
#include "cache.h"
#include "psziernyo.h"
#include "ul.h"

extern int main(int argc, char * argv[])
{
    t_forras * forras = NULL;
    t_forras_type forras_type = forras_type_none;
    t_fordulo * fordulo = NULL;
    char name[NAME_LEN];
    int i = 0;
    int j = 0;
    int k = 0;
    char time_string[TIME_MAXLEN];
    struct tm * tmnow;
    struct timeval tstart;
    struct timeval tend;
    signed int diff;
    int ret;
    int cache_ret;
    
#ifdef DEBUG
    debug_open(DEBUGFILE);
#endif
#ifdef USE_CHECK_ALLOC
    if (check_open_file(ALLOCFILE) < 0)
    {
#ifdef DEBUG
	debug(fordulo, "memory allocation checking could not open the output file \"%s\"\n", ALLOCFILE);
#endif
	return -1;
    }
    else
    {
#ifdef DEBUG
	debug(fordulo, "memory allocation checking filename=\"%s\"\n",ALLOCFILE);
#endif
    }
#endif
    
    gettimeofday(&tstart, NULL);
    if (!(tmnow = localtime(&tstart.tv_sec)))
	strcpy(time_string, "?");
    else
	strftime(time_string, TIME_MAXLEN, "%H:%M:%S", tmnow);
    

    init_program();
    cache_job_init();
    ret = read_commandline(argc, argv);
    
    if (ret == 0)
    {
	kiir(fordulo, "Start time: %s.%02d\r\n", time_string, tstart.tv_usec / 10000);
	init_conf();
	init_txtfiles();
	cache_ret = 0;
	if (get_pszi_mode())
	{
	    dataflow_set(fordulo_output_pszi);
	}
	else
	{
	    cache_ret = cache_job_proc();
	}
	
	for (i = 1, k=0, j=0; (cache_ret == 0) && (i < argc); i++)
	{
#ifdef DEBUG
	    debug(fordulo, "main: argv[%d]=\"%s\"\n", i, argv[i]);
#endif
	    if (argv[i] == NULL)
	    {
	        continue;
	    }
	    if (*argv[i] == '-')
	    {
		continue;
	    }
	    
	    j++;
	    
	    if (strlen(argv[i]) < NAME_LEN)
	    {
		strcpy(name,argv[i]);
#ifdef DEBUG
		debug(fordulo, "main: fordulo load start: %s\n", name);
#endif
		forras_type = forras_type_file;
		if (forras = forras_create(name, forras_type))
		{
		    forras->n = j;
		    if (get_pszi_mode())
		    {
			fordulo = fordulo_load(forras);
		    }
		    else
		    {
			fordulo = fordulo_load(forras);
		    }
		}
	    }
	    else
	    {
		strcpy(name,"");
#ifdef DEBUG
		debug(fordulo, "main: fordulo load a name (%s) tul hosszu, hosszabb mint %d karakter\n", name, NAME_LEN);
#endif
		fordulo = NULL;
	    }

	    if (fordulo)
	    {
#ifdef DEBUG
		debug(fordulo, "main: fordulo reformat start: %d_%d\n", fordulo->karakter->ksz, fordulo->fsz);
#endif
		if (get_pszi_mode())
		{
		    if (psziernyo_main(fordulo) > 0)
		    {
		        k++;
		    }
		}
		else
		{
		    cache_load(fordulo);
		    search_init(fordulo);
		    spell_figyelt_init();
		    karakter_targy_figyelt_init(fordulo->karakter);
		    if (reformat(fordulo) > 0)
		    {
			k++;
		    }
		    torony_unload();
#ifdef DEBUG
		    debug(fordulo, "main: fordulo reformat end\n");
#endif
		    cache_save(fordulo);
#ifdef DEBUG
		    debug(fordulo, "main: cache saved\n");
#endif
		    cache_unload(fordulo);
#ifdef DEBUG
		    debug(fordulo, "main: cache unloaded\n");
#endif
		    if (aconf_get_curr("winul_nextul") == 1)
		    {
			ultemp(fordulo);
#ifdef DEBUG
			debug(fordulo, "main: tempul wroted\n");
#endif
		    }
		}
		fordulo_unload(fordulo);
		fordulo = NULL;
#ifdef DEBUG
		debug(fordulo, "main: fordulo unload done\n");
#endif
	    }
	    else
	    {
#ifdef DEBUG
		debug(fordulo, "main: nem sikerult beolvasni: \"%s\"\n",name);
#endif
	    }
	    forras_destroy(forras);
	    forras = NULL;
	}

	deinit_txtfiles();
	conf_unload();

	gettimeofday(&tend, NULL);
	if (!(tmnow = localtime(&tend.tv_sec)))
	    strcpy(time_string, "?");
	else
	    strftime(time_string, TIME_MAXLEN, "%H:%M:%S", tmnow);
	
	kiir(fordulo, "\r\nEnd time: %s.%02d\r\n", time_string, tend.tv_usec / 10000);
	diff = tend.tv_usec / 10000 - tstart.tv_usec / 10000;
	if (get_pszi_mode())
	{
	    kiir(fordulo, "%d fordulo, vizsgalva ebbol %d, %d.%02d masodperc alatt (%lu output, c=%lu)\r\n\r\n", j, k, tend.tv_sec - tstart.tv_sec - (diff < 0 ? 1 : 0), (diff < 0 ? 100+diff : diff), get_fpn(), get_fpc());
	}
	else if (cache_ret)
	{
	    kiir(fordulo, "%d fordulo cachelve %d.%02d masodperc alatt\r\n\r\n", cache_ret, tend.tv_sec - tstart.tv_sec - (diff < 0 ? 1 : 0), (diff < 0 ? 100+diff : diff));
	}
	else
	{
	    kiir(fordulo, "%d fordulo, konvertalva ebbol %d, %d.%02d masodperc alatt (%lu output, c=%lu)\r\n\r\n", j, k, tend.tv_sec - tstart.tv_sec - (diff < 0 ? 1 : 0), (diff < 0 ? 100+diff : diff), get_fpn(), get_fpc());
	}
    }
    
    deinit_program();
    if (!get_pszi_mode())
    {
	cache_job_destroy();
    }
    deinit_globals();

#ifdef USE_CHECK_ALLOC
    check_cleanup();
#ifdef DEBUG
    debug(fordulo, "memory allocation checking cleanup\n");
#endif
#endif
#ifdef DEBUG
    debug_close();
#endif
#ifdef WIN32
    if (!get_nopause())
    {
	kiir(fordulo, "Bezarashoz nyomj egy ENTER-t.\r\n");
	getchar();
    }
#endif
}
