/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#define SORSZAM_INTERNAL_ACCESS
#include "include.h"
#include "sorszam.h"
#include "fordulo.h"
#include "globals.h"

static t_sorszam_kivetel sorszam_kivetelek[] = 
{
    {217, "massza", NULL},
    {220, "palotar", "kirlyi palotar"},
    {242, "qwarg rohamosztagos", "quwarg rohamosztagos"},
    {317, "mpat fatty", "mpat fatty"},
    {335, "kufr", "Thargodan kufr"},
    {409, "vzbelt ryuku", "vzbelt ryuku kalz"},
    {423, "a csontvz hadr", "csontvz hadr"},
    {424, "a ksrtetvarzsl", "ksrtetvarzsl"},
    {630, "fertztt dr.lrva ", "fertztt drakolderlrva "},
    {681, "trpe knzmester", "stt trpe knzmester"},
    {682, "trpe kincstros", "stt trpe kincstros"},
    {683, "trpe hadvezr", "stt trpe hadvezr"},
    {693, "vark. kardtncos", "varkaudar kardtncos"},
    {699, "haprszag thargodan", "hapralszag Thargodan"},
    {701, "thargodan raktfnk ", "Thargodan raktrfnk"},
    {743, "zt'tuzzarxn", "Zt'tuzzarxn"},
    {0, NULL, NULL}
};

extern int nev_kivetel_check(char * nn, char ** pontosnev)
{
    int n1;
    int n2;
    int ret = 0;
    int i;
    
#ifdef DEBUG
    debug(NULL, "nev_kivetel_check: nn=\"%s\"\n",nn);
#endif
    for (i = 0; (i < (sizeof(sorszam_kivetelek) / sizeof(t_sorszam_kivetel))) && sorszam_kivetelek[i].nev; i++)
    {
	n1 = strlen(sorszam_kivetelek[i].nev);
	n2 = strlen(nn);
	if ((strstr_(sorszam_kivetelek[i].nev,nn,NULL)) && (n1 - n2 <= 3) && (n1 - n2 >= -2))
	{
	    ret = sorszam_kivetelek[i].szam;
	    if (sorszam_kivetelek[i].jonev)
	    {
		*pontosnev = strdup(sorszam_kivetelek[i].jonev);
	    }
	    else
	    {
		*pontosnev = NULL;
	    }
	    break;
	}
    }
    return ret;
}

extern char * nev_vege_check(char * nn, char * p2)
{
    int val = 0;
    
    if (!p2 || !nn || (p2 < nn))
    {
	return NULL;
    }

#ifdef DEBUG
    debug(NULL, "nev_vege_check 1: nn=\"%.*s\", *p2=\"%c\"\n", p2 - nn,nn, *p2);
#endif
    if ((p2 - nn > 4) && (*p2 == '.'))
    {
	p2--;
    }
    if ((p2 - nn > 4) && (*p2 == 'l'))
    {
	val = 1;
	p2--;
	if ((p2 - nn > 3) && ((*p2 == 'e') || (*p2 == 'a')))
	{
	    p2--;
	    if (*p2 == 'v')
	    {
		p2 -= 2;
	    }
	    else if (*p2 == *(p2-1))
	    {
		p2--;
	    }
	    else if (*(p2-1) == *(p2-2))
	    {
		p2--;
		*p2 = *(p2+1);
	    }
	}
    }
    else if ((p2 - nn > 4) && (*p2 == 't'))
    {
	if (*(p2-1) == '')
	{
	    p2 -= 1;
	}
	else
	{
	    p2 -= 3;
	}
#ifdef DEBUG
	debug(NULL, "nev_vege_check targyrag takaritas: nn - p2 = %d, nn=\"%.*s\"\n", p2 - nn, p2 - nn, nn);
#endif
	if (!val && (p2 - nn > 4))
	{
	    if ((*(p2) == 'a') || (*(p2) == 'e'))
	    {
#ifdef DEBUG
		debug(NULL, "nev_vege_check targyrag takaritas, abbol lesz utolso 1 levagva: nn=\"%.*s\"\n", p2 - nn, nn);
#endif
		p2 -= 1;
	    }
	}
    }
    if ((p2 - nn > 2) && (*p2 == '-'))
    {
	p2--;
    }
    else if ((p2 - nn > 3) && (*(p2-1) == '-'))
    {
	p2 -= 2;
    }
    else if ((p2 - nn > 4) && (*(p2-2) == '-'))
    {
	p2 -= 3;
    }
    else if ((p2 - nn > 5) && (*(p2-3) == '-'))
    {
	p2 -= 3;
    }
    p2++;
    while ((*(p2-1) == ' ') && (p2 > nn))
    {
	p2--;
    }
    *p2 = '\0';
#ifdef DEBUG
    debug(NULL, "nev_vege_check 2: nn=\"%s\"\n",nn);
#endif
}

int karakter_check_szam(t_fordulo * fordulo, char * nev, char ** pontosnev)
{
    char **p1;
    char ** p2;
    char * t1;
    char * t2;
    char * t3;
    char * nn;
    char szn[63];
    char tmp;
    int ret = 0;
        
    if (!fordulo || !fordulo->nekikirhatsz || !nev)
    {
	return 0;
    }

    t1 = skip_space(nev);

    if (!(nn = strdup(t1)))
    {
	return ret;
    }
#ifdef DEBUG
    debug(NULL, "karakter_check_szam: nn=\"%s\"\n",nn);
#endif

    str_tolower2(nn);
    t2 = nn + strlen(nn) - 1;

    nev_vege_check(nn, t2);

    p1 = &fordulo->nekikirhatsz;
    p2 = p1 + 1;

    while ((*p2 == NULL) &&  (p2 < &fordulo->end))
    {
	p2++;
    }
    tmp = **p2;
    **p2 = '\0';
    
    for (t2 = *p1; (t1 = strstr2(t2, "class=\"t1\">")) && (t2 = strstr(t1, ":")); )
    {
#ifdef DEBUG
    debug(NULL, "karakter_check_szam: t1=\"%.*s\"\n",t2 - t1, t1);
#endif
	if ((t3 = strstr2(t1, "(#")) && (t3 < t2) && (t3 - t1 < 62))
	{
	    sprintf(szn,"%.*s", t3 - t1 - 3, t1);
	    str_tolower2(szn);
#ifdef DEBUG
    debug(NULL, "karakter_check_szam: nn=\"%s\", szn=\"%s\"\n", nn, szn);
#endif
	    if (strstr_(szn, nn, NULL))
	    {
		if (isdigit2(*t3))
		{
		    ret = atoi(t3);
		    sprintf(szn,"%.*s", t3 - t1 - 3, t1);
		    *pontosnev = strdup(szn);
		    break;
		}
	    }
	}
	
    }
    **p2 = tmp;
    if (nn)
    {
	free(nn);
    }
    return ret;
}

extern int karakter_get_szam(char * nev, char ** pontosnev)
{
    char * p1 = NULL;
    char * p2 = NULL;
    char * p3 = NULL;
    char * p4 = NULL;
    char * p5 = NULL;
    char * ke = NULL;
    char * nn = NULL;
    char * mm = NULL;
    char * oo = NULL;
    int ret = 0;
    int i;
    int megvan = 0;
    int change = 0;
    char * szn;
    
    if (pontosnev)
    {
	*pontosnev = NULL;
    }
    if (!nev)
    {
	return ret;
    }
    
    p1 = skip_space(nev);
    
    if (!(nn = strdup(p1)) || !(oo = strdup(p1)))
    {
	return ret;
    }
#ifdef DEBUG
    debug(NULL, "karakter_get_szam: nn=\"%s\"\n",nn);
#endif
    str_tolower2(nn);
    str_tolower2(oo);
    p2 = nn + strlen(nn) - 1;

    nev_vege_check(nn, p2);

    if (p3 = karakters_str_get())
    {
	for (ke = p3 + strlen(p3); (p2 = strstr2_i(p3, nn, NULL)) && (p2 < ke); )
	{
	    for ( ; !isdigit2(*p2) && (p2 > p3); p2--);
	    for (p1 = p2; isdigit2(*p1); p1--);
	    p1++;
	    for ( p2++; (*p2 == ' ') || (*p2 == '\t') ||(*p2 == '\r') ||(*p2 == '\n'); p2++);
	
	    if (!(p3 = strstr(p2, "\n")))
	    {
#ifdef DEBUG
		debug(NULL, "karakter_get_szam: break, nincs \\n benne p2=\"%.30s\"\n",p2);
#endif
		break;
	    }
	    for ( ; *(p3-1) == '\r' ||  *(p3-1) == '\n' || *(p3-1) == '\t' || *(p3-1) == ' '; p3--);
	
	    if (!(mm = malloc(p3 - p2 + 1)))
	    {
		break;
	    }
	    sprintf(mm, "%.*s",p3-p2, p2);
#ifdef DEBUG
	    debug(NULL, "karakter_get_szam: nn=\"%s\", mm=\"%s\", oo=\"%s\"\n",nn,mm, oo);
#endif

	    if (strstr2_i(oo, mm, NULL) != NULL)
	    {
		megvan = 1;
	    }
	    else
	    {
		p5 = oo + strlen(oo) - 1;
		if ((*p5 == 't'))
		{
		    if ((*(p5-1) == 'a') && (*(p5-3) == 'a'))
		    {
			change = 1;
			*(p5-3) = '';
		    }
		    else if ((*(p5-1) == 'a') && (*(p5-4) == 'a'))
		    {
			change = 1;
			*(p5-4) = '';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-3) == 'e'))
		    {
			change = 1;
			*(p5-3) = '';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-4) == 'e'))
		    {
			change = 1;
			*(p5-4) = '';
		    }
		}
		else if ((*p5 == 'l'))
		{
		    if ((*(p5-1) == 'a') && (*(p5-3) == ''))
		    {
			change = 1;
			*(p5-3) = 'a';
		    }
		    else if ((*(p5-1) == 'a') && (*(p5-4) == ''))
		    {
			change = 1;
			*(p5-4) = 'a';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-3) == ''))
		    {
			change = 1;
			*(p5-3) = 'e';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-4) == ''))
		    {
			change = 1;
			*(p5-4) = 'e';
		    }
		}
		if (change)
		{
#ifdef DEBUG
		    debug(NULL, "karakter_get_szam: change nn=\"%s\", mm=\"%s\", oo=\"%s\"\n",nn,mm, oo);
#endif
		    if (strstr2_i(oo, mm, NULL) != NULL)
		    {
			megvan = 1;
		    }
		}
	    }
	    if (megvan)
	    {
#ifdef DEBUG
		debug(NULL, "karakter_get_szam: p1=\"%.*s\"\n",p3-p1,p1);
#endif
		if (isdigit2(*p1))
		{
#ifdef DEBUG
		    debug(NULL, "karakter_get_szam: p1 isdigit2=\"%.*s\"\n",p3-p1,p1);
		    debug(NULL, "karakter_get_szam: nn=\"%s\", mm=\"%s\"\n",nn,mm);
#endif

/*		    if ((strlen(nn) == strlen(mm)) || (strlen(nn) + 1 == strlen(mm)))
		    {
			ret = atoi(p1);
			if (szn = strdup(mm))
			{
			    if (pontosnev)
			    {
				*pontosnev = szn;
			    }
			}
			free(mm);
			break;
		    }
*/
		    if (!ret)
		    {
			ret = atoi(p1);
			if (pontosnev && (szn = strdup(mm)))
			{
			    *pontosnev = szn;
			}
		    }
		    else 
		    {
			ret = -1;
			if (pontosnev)
			{
			    if (*pontosnev)
			    {
				free(*pontosnev);
			    }
			    *pontosnev = NULL;
			}
		    }
		}
	    }
#ifdef DEBUG
	    debug(NULL, "karakter_get_szam: p2=\"%.*s\"\n",p3-p1, p1);
#endif
	    free(mm);
	}
    }
#ifdef DEBUG
    debug(NULL, "karakter_get_szam: ret=\"%d\"\n",ret);
#endif
    if (ret == -1)
    {
	ret = 0;
    }
    if (nn)
    {
	free(nn);
    }
    if (oo)
    {
	free(oo);
    }
    return ret;
}

extern int monster_get_szam(char * nev, char ** pontosnev)
{
    char * p1 = NULL;
    char * p2 = NULL;
    char * p3 = NULL;
    char * p4 = NULL;
    char * p5 = NULL;
    char * ke = NULL;
    char * nn = NULL;
    char * mm = NULL;
    char * oo = NULL;
    int ret = 0;
    int i;
    int megvan = 0;
    int change = 0;
    char * szn;
    char tmp;

    *pontosnev = NULL;
    if (!nev)
    {
	return ret;
    }

    p1 = skip_space(nev);

    if ((*p1 == 'a') && (*(p1+1) == 'z') && (*(p1+2) == ' '))
    {
	p1 += 2;
	p1 = skip_space(p1);
    }
    if (strncmp(p1, "csapat", 6) == 0)
    {
	p1 += 6;
	p1 = skip_space(p1);
    }
    if (!(nn = strdup(p1)) || !(oo = strdup(p1)))
    {
	return ret;
    }
#ifdef DEBUG
    debug(NULL, "monster_get_szam: nn=\"%s\"\n",nn);
#endif
    str_tolower2(nn);
    str_tolower2(oo);
    p2 = nn + strlen(nn) - 1;

    nev_vege_check(nn, p2);

    if (ret = nev_kivetel_check(nn, pontosnev))
    {
// Nem csinalunk semmit, csak at kell ugrani az else if agat :)
    }
    else if (p3 = monsters_str_get())
    {
	for (ke = p3 + strlen(p3); (p2 = strstr2_i(p3, nn, NULL)) && (p2 < ke); )
	{
	    for ( ; !isdigit2(*p2) && (p2 > p3); p2--);
	    for (p1 = p2; isdigit2(*p1); p1--);
	    p1++;
	    for ( p2++; (*p2 == ' ') || (*p2 == '\t') ||(*p2 == '\r') ||(*p2 == '\n'); p2++);
	
	    if (strncmp(p2, "csapat", 6) == 0)
	    {
		p2 += 6;
		p2 = skip_space(p2);
	    }
	    if (!(p3 = strstr(p2, "\n")))
	    {
#ifdef DEBUG
		    debug(NULL, "monster_get_szam: break, nincs \\n benne p2=\"%.30s\"\n",p2);
#endif
		    break;
	    }
	    for ( ; *(p3-1) == '\r' ||  *(p3-1) == '\n' || *(p3-1) == '\t' || *(p3-1) == ' '; p3--);
	
	    if (!(mm = malloc(p3 - p2 + 1)))
	    {
		break;
	    }
	    sprintf(mm, "%.*s",p3-p2, p2);
#ifdef DEBUG
	    debug(NULL, "monster_get_szam: nn=\"%s\", mm=\"%s\", oo=\"%s\"\n",nn,mm,oo);
#endif
	    if (strstr2_i(oo, mm, NULL) != NULL)
	    {
		megvan = 1;
	    }
	    else
	    {
		p5 = oo + strlen(oo) - 1;
		if (*p5 == 't')
		{
		    if ((*(p5-1) == 'a') && (*(p5-3) == 'a'))
		    {
			change = 1;
			*(p5-3) = '';
		    }
		    else if ((*(p5-1) == 'a') && (*(p5-4) == 'a'))
		    {
			change = 1;
			*(p5-4) = '';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-3) == 'e'))
		    {
			change = 1;
			*(p5-3) = '';
		    }
		    else if ((*(p5-1) == 'e') && (*(p5-4) == 'e'))
		    {
			change = 1;
			*(p5-4) = '';
		    }
		    if (change)
		    {
			if (strstr2_i(oo, mm, NULL) != NULL)
			{
			    megvan = 1;
			}
		    }
		    if (!megvan)
		    {
			tmp = *(p5-1);
			*(p5-1) = *(p5-2);
			*(p5-2) = tmp;
			if (strstr2_i(oo, mm, NULL) != NULL)
			{
			    megvan = 1;
			}
		    }
		}
	    }
	    if (megvan)
	    {
#ifdef DEBUG
		debug(NULL, "monster_get_szam: p1=\"%.*s\"\n",p3-p1,p1);
#endif
		if (isdigit2(*p1))
		{
#ifdef DEBUG
		    debug(NULL, "monster_get_szam: p1 isdigit2=\"%.*s\"\n",p3-p2,p1);
#endif
		    if ((strlen(nn) == strlen(mm)) || (strlen(nn) + 1 == strlen(mm)) || (strlen(oo) == strlen(mm)))
		    {
			ret = atoi(p1);
/*
			if (szn = strdup(mm))
			{
			    *pontosnev = szn;
			}
*/
			free(mm);
			break;
		    }
		    else if (!ret)
		    {
			ret = atoi(p1);
/*
			if (szn = strdup(mm))
			{
			    *pontosnev = szn;
			}
*/
		    }
		    else 
		    {
			ret = -1;
/*
			if (*pontosnev)
			{
			    free(*pontosnev);
			}
			*pontosnev = NULL;
*/
		    }
		}
	    }
#ifdef DEBUG
	    debug(NULL, "monster_get_szam: p2=\"%.*s\"\n",p3-p1, p1);
#endif
	    free(mm);
	}
    }
#ifdef DEBUG
    debug(NULL, "monster_get_szam: ret=\"%d\"\n",ret);
#endif
    if (ret == -1)
    {
	ret = 0;
    }

    if (nn)
    {
	free(nn);
    }
    if (oo)
    {
	free(oo);
    }

    return ret;
}

extern int init_karakters(char * filename)
{
    FILE * fp;
    struct stat kfile;
    unsigned long size;
    char * karakters;
#ifdef DEBUG
    debug(NULL, "init_karakters start\n");
#endif
    if (filename && (stat(filename, &kfile) == 0))
    {
#ifdef DEBUG
	debug(NULL, "init_karakters filesize\n");
#endif
	size = (unsigned long)kfile.st_size + 1;
	if (karakters = malloc(size))
	{
	    memset(karakters,0,size);
	    if(fp = fopen(filename,"rb"))
	    {
#ifdef DEBUG
		debug(NULL, "init_karakters open\n");
#endif
		if(fread(karakters,1,size,fp))
		{
#ifdef DEBUG
		    debug(NULL, "init_karakters read\n");
#endif
		    fclose(fp);
		    str_tolower2(karakters);
		    karakters_str_set(karakters);
		    free(karakters);
		    return 1;
		}
	    }
	    free(karakters);
	}
    }
#ifdef DEBUG
    debug(NULL, "init_karakters none\n");
#endif
    return 0;
}

extern int init_monsters(char * filename)
{
    FILE * fp;
    struct stat kfile;
    unsigned long size;
    char * monsters;
#ifdef DEBUG
    debug(NULL, "init_monsters start\n");
#endif
    if (filename && (stat(filename, &kfile) == 0))
    {
#ifdef DEBUG
	debug(NULL, "init_monsters filesize\n");
#endif
	size = (unsigned long)kfile.st_size + 1;
	if (monsters = malloc(size))
	{
	    memset(monsters,0,size);
	    if(fp = fopen(filename,"rb"))
	    {
#ifdef DEBUG
		debug(NULL, "init_monsters open\n");
#endif
		if(fread(monsters,1,size,fp))
		{
#ifdef DEBUG
		    debug(NULL, "init_monsters read\n");
#endif
		    fclose(fp);
		    str_tolower2(monsters);
		    monsters_str_set(monsters);
		    free(monsters);
		    return 1;
		}
	    }
	    free(monsters);
	}
    }
#ifdef DEBUG
    debug(NULL, "init_monsters none\n");
#endif
    return 0;
}

