/*
 * Copyright (C) 2003  Vizi Gabor (vizi@csucs.net)
 *
 * Ez az adatallomany a reformat program forraskodjanak resze.
 *
 * A program hasznalata, masolasa, terjesztese maganszemely 
 * bevetelt nem eredmenyezo celjara az eredeti forras, 
 * a szerzoi jogi megjegyzes es a hasznalati feltetelek 
 * feltuntetesevel es valtozatlanul hagyasaval, szabadon lehetseges. 
 * A program minden garancia nelkul kerul kozreadasra, a program
 * mukodesebol fakado osszes kockazat a felhasznalot terheli. 
 */
#ifdef WIN32
#define WIN_INTERNAL_ACCESS
#include "include.h"
#include "win.h"
#include "conf.h"
#include <windef.h>
#include <windows.h>
#include <winreg.h>
#include <winbase.h>
#include <io.h>

#define WIN_BUF_LEN 4096
#define WIN_USER_CONF_DIR_NAME "ReFormat"

extern int init_win(void)
{
    HKEY K, K1;
    DWORD Val;
    char buf[WIN_BUF_LEN];
    LONG l;
    LONG type;
    LONG r;
    char * genv;
    char * dir;
    char * p;
    
    if ((dir = getenv("USERPROFILE")) && (strlen(dir) > 3))
    {
	if (p = get_filename(dir, WIN_USER_CONF_DIR_NAME))
	{
	    bconf_set_value("confdir", conftype_user, p);
	    free(p);
	}
    }
    
    sprintf(buf,"");
    l = WIN_BUF_LEN;
    type = REG_SZ;
    
    r = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ReFormat", 0, KEY_READ, &K);
    if (r == ERROR_SUCCESS)
    {
	RegQueryValueEx(K, "InstallDir", 0, &type, (BYTE*)buf, &l);
	RegCloseKey(K);
	bconf_set_value("confdir", conftype_global, buf);
    }
}

extern int wdir(char * path, char * mask, char *** result)
{
    char * wpath;
    t_wdir * wdir;
    int i, n;
    char * name;
    char ** res;
    int ret;
    int hiba = 0;

    if (!mask)
    {
	mask = "*";
    }
    if (strlen(path)+strlen(mask)+2 > _MAX_PATH)
    {
	return -1;
    }
    
    if (!(wpath = get_filename(path, mask)))
    {
	return -1;
    }
    
    if (!(wdir = malloc(sizeof(t_wdir))))
    {
	return -1;
    }
    wdir->wpath = wpath;
    memset(&wdir->fileinfo, 0, sizeof(wdir->fileinfo));
    
    wdir->lFindHandle = _findfirst(wdir->wpath, &wdir->fileinfo);
    if (wdir->lFindHandle < 0)
    {
#ifdef DEBUG
	debug(NULL, "wdir: unable to open directory \"%s\" for reading (_findfirst: %s\n)", wdir->wpath, strerror(errno));
#endif
	free((void *)wdir->wpath);
	free(wdir);
	return -1;
    }
    
    for (n = 0, hiba = 0; hiba == 0; n++, hiba = _findnext(wdir->lFindHandle, &wdir->fileinfo));
    _findclose(wdir->lFindHandle);
    memset(&wdir->fileinfo, 0, sizeof(wdir->fileinfo));

    wdir->lFindHandle = _findfirst(wdir->wpath, &wdir->fileinfo);
    if (wdir->lFindHandle < 0)
    {
#ifdef DEBUG
	debug(NULL, "wdir: unable to open directory \"%s\" for reading (_findfirst: %s\n)", wdir->wpath, strerror(errno));
#endif
	free((void *)wdir->wpath);
	free(wdir);
	return -1;
    }
    
    if (!(res = malloc(sizeof(char *) * n)))
    {
	free((void *)wdir->wpath);
	free(wdir);
	return -1;
    }
    memset(res, 0, sizeof(char *) * n);

    for (i = 0, hiba = 0; (hiba == 0) && (i < n); i++, hiba = _findnext(wdir->lFindHandle, &wdir->fileinfo))
    {
	if (!(name = strdup(wdir->fileinfo.name)))
	{
	    hiba = 1;
	    break;
	}
	
	res[i] = name;
    }
    _findclose(wdir->lFindHandle);
    
    if (hiba == 1)
    {
	for(i = 0; i < n; i++)
	{
	    if (res && res[i])
	    {
		free(res[i]);
	    }
	}
	if (res)
	{
	    free(res);
	}
	res = NULL;
	n = -1;
    }
    
    free((void *)wdir->wpath);
    free(wdir);
    
    *result = res;
    return n;
}

#endif
